from cli.utils.logger import verbose
import click
import json
import sys
from ..utils.run_query_pid_by_packagename import query_pid_by_packagename


@click.command("query-pid")
@click.argument("package_names", type=click.STRING, nargs=-1, required=True)
@click.option("--json-output", is_flag=True, help="Output results in JSON format")
def query_pid(package_names, json_output):
    """
    Check if the processes for the specified package names are running.

    PACKAGE_NAMES: One or more package names to check.
    Returns the running status for each package name. If the --json-output option is used, outputs the result in JSON format.
    """
    try:
        # Get the running status for each package name
        status_dict = query_pid_by_packagename(list(package_names))

        if json_output:
            # Output results in JSON format
            print(json.dumps(status_dict))
        else:
            # Output results in plain text format
            for package_name, is_running in status_dict.items():
                status = "Running" if is_running else "Not running"
                verbose(f"Package name '{package_name}': {status}")

        # Check if any process is running
        any_running = any(status_dict.values())
        exit(0 if any_running else 1)
    except Exception as e:
        print(f"Error occurred while executing query-pid command: {e}", file=sys.stderr)
        exit(1)  # Return 1 to indicate an error



import json
import os
from cli.utils.logger import verbose
import questionary
from rich import print as verbose
from typing import Any, Dict, List, Optional, Set, Tuple, TypeVar
from ..config.app_config import APP_ENV


# Import proper types from registry module
from ..types.registry import (
    ConnectionDetails, RegistryServer,
    ServerConfig, ConfiguredServer, ConfigSchemaProperty
)


def convert_value_to_type(value: Any, type_str: Optional[str]) -> Any:
    """Converts a value to the specified type string."""
    if not type_str:
        return value

    def invalid(expected: str):
        raise ValueError(f"Invalid {expected} value: {json.dumps(value)}")

    try:
        if type_str == "boolean":
            str_val = str(value).lower()
            if str_val == "true":
                return True
            if str_val == "false":
                return False
            invalid("boolean")
        elif type_str == "number":
            return float(value)
        elif type_str == "integer":
            return int(value)
        elif type_str == "string":
            return str(value)
        elif type_str == "array":
            if isinstance(value, list):
                return value
            return [v.strip() for v in str(value).split(",")]
        else:
            return value
    except (ValueError, TypeError):
        invalid(type_str)


def format_config_values(
    connection: ConnectionDetails,
    config_values: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """Formats and validates config values based on the connection's schema."""
    formatted_values: Dict[str, Any] = {}
    missing_required: List[str] = []
    config_values = config_values or {}

    schema = connection.configSchema or {}
    env = schema.env if schema else {}
    if not env:
        return config_values

    required: Set[str] = set(schema.required if schema else [])

    for property_name, prop_details in env.items():
        value = config_values.get(property_name)
        prop_type = prop_details.type

        if value is None:
            if property_name in required:
                missing_required.append(property_name)
            else:
                formatted_values[property_name] = prop_details.default
        else:
            try:
                formatted_values[property_name] = convert_value_to_type(value, prop_type)
            except ValueError as e:
                # If conversion fails but it's not required, set to default or None
                if property_name not in required:
                    formatted_values[property_name] = prop_details.default
                else:
                    # Re-raise or handle error if conversion fails for a required field
                    # For now, let's add to missing required to indicate an issue
                    verbose(f"Warning: Could not convert required value for {property_name}: {e}")
                    missing_required.append(f"{property_name} (invalid format)")

    if missing_required:
        raise ValueError(
            f"Missing or invalid required config values: {', '.join(missing_required)}"
        )

    return formatted_values


def validate_config(
    connection: ConnectionDetails,
    saved_config: Optional[Dict[str, Any]] = None,
) -> Tuple[bool, Optional[Dict[str, Any]]]:
    """Validates saved config against the connection schema."""
    schema = connection.configSchema or {}
    env = schema.env if schema else {}
    if not env:
        return True, {}

    saved_config = saved_config or {}

    try:
        formatted_config = format_config_values(connection, saved_config)
        required = set(schema.required if schema else [])
        has_all_required = all(
            key in formatted_config and formatted_config[key] is not None
            for key in required
        )
        return has_all_required, formatted_config
    except ValueError:
        # Attempt to return partially valid config if formatting fails
        try:
            partial_config = {
                k: v for k, v in saved_config.items() if v is not None
            }
            # Check if the partial config *still* satisfies required fields *that are present*
            # This is tricky, maybe just return False and the original partial config
            return False, partial_config
        except Exception:
            return False, None


async def prompt_for_config_value(
    key: str,
    schema_prop: ConfigSchemaProperty,
    required: Set[str],
) -> Any:
    """Prompts the user for a single config value based on schema property."""
    is_required = key in required
    required_text = " [red](required)[/red]" if is_required else " (optional)"
    message = f"{schema_prop.description or f'Enter value for {key}'}{required_text}"
    prop_type = schema_prop.type
    default_value = schema_prop.default

    # Determine questionary prompt type
    if key.lower().endswith("key") or key.lower().endswith("secret") or key.lower().endswith("token"):
        prompt_method = questionary.password
    elif prop_type == "boolean":
        prompt_method = questionary.confirm
        # questionary confirm default needs bool or None
        if default_value is not None:
            default_value = str(default_value).lower() == 'true'
        else:
            default_value = None  # Or False if you prefer
    elif prop_type == "number" or prop_type == "integer":
        # Using text prompt with validation for numbers
        prompt_method = questionary.text
    elif prop_type == "array":
        message += " (comma-separated)"
        prompt_method = questionary.text
    else:
        prompt_method = questionary.text

    # Validation function
    def validate_input(text: str) -> bool | str:
        if is_required and not text:
            return "This field is required."
        if prop_type == "number" or prop_type == "integer":
            try:
                if prop_type == "integer":
                    int(text)
                else:
                    float(text)
                return True
            except ValueError:
                return "Please enter a valid number."
        return True

    # Use appropriate prompt method
    if prompt_method == questionary.confirm:
        # Handle confirm separately as it doesn't take validate
        # Note: questionary.confirm returns True/False directly
        # We need to handle the case where the user might skip an optional confirm
        # For simplicity, let's assume confirm always gets an answer if prompted
        answer = await prompt_method(message, default=default_value).ask_async()
        # If it's required and they somehow skip (e.g., Ctrl+C), handle appropriately
        if is_required and answer is None:
            verbose("[bold red]Required field cannot be skipped.[/bold red]")
            # Re-prompt or exit - for now, return None which might fail later validation
            return None
        return answer
    else:
        # For text, password
        q = prompt_method(
            message,
            default=str(default_value) if default_value is not None else "",
            validate=validate_input
        )
        result = await q.ask_async()
        if result is None and is_required:
            verbose("[bold red]Required field cannot be skipped.[/bold red]")
            return None  # Or raise an error / re-prompt
        # Convert back if needed (e.g., for numbers, arrays)
        if result is not None:
            try:
                return convert_value_to_type(result, prop_type)
            except ValueError:
                verbose(
                    f"[yellow]Warning: Could not convert input '{result}' to type '{prop_type}'. Using raw input.[/yellow]")
                return result  # Return raw input if conversion fails after prompt
        return result  # Return None if skipped (and not required)


async def collect_config_values(
    connection: ConnectionDetails,
    existing_values: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """Collects config values from existing values or user prompts."""
    schema = connection.configSchema or {}
    env = schema.env if schema else {}
    if not env:
        return {}

    base_config: Dict[str, Any] = {}

    # Validate existing values first
    if existing_values:
        is_valid, validated_config = validate_config(connection, existing_values)
        if is_valid and validated_config is not None:
            return validated_config
        # Use partially validated config as base if validation failed
        base_config = validated_config or {}

    required = set(schema.required if schema else [])

    verbose("[bold blue]Please provide the following configuration values:[/bold blue]")

    for key, prop_details in env.items():
        # Skip if value already exists and is not None in the base_config
        if key in base_config and base_config[key] is not None:
            continue

        # Prompt if missing
        value = await prompt_for_config_value(key, prop_details, required)

        # Handle skipped required fields after prompt
        if value is None and key in required:
            # This case should ideally be handled within prompt_for_config_value
            # but as a fallback:
            verbose(f"[bold red]Error: Required configuration '{key}' was not provided.[/bold red]")
            raise ValueError(f"Missing required configuration: {key}")

        # Assign even if None (for optional fields skipped)
        base_config[key] = value

    # Final format and validation after collecting all values
    try:
        return format_config_values(connection, base_config)
    except ValueError as e:
        verbose(f"[bold red]Configuration error:[/bold red] {e}")
        # Return the collected (potentially incomplete/invalid) config
        # or raise the error depending on desired strictness
        return base_config


def choose_stdio_connection(connections: List[ConnectionDetails]) -> Optional[ConnectionDetails]:
    """Chooses the best stdio connection from a list based on priority."""
    stdio_connections = [conn for conn in connections if conn.type == "stdio"]
    if not stdio_connections:
        return None

    priority_order = ["npx", "uvx", "docker"]  # Assuming similar priority in Python context

    # Prioritize published connections first
    for priority in priority_order:
        for conn in stdio_connections:
            stdio_func = conn.stdioFunction
            if (
                conn.published
                and isinstance(stdio_func, list)
                and stdio_func
                and isinstance(stdio_func[0], str)
                and stdio_func[0].startswith(priority)
            ):
                return conn

    # If no published connection found, check non-published
    for priority in priority_order:
        for conn in stdio_connections:
            stdio_func = conn.stdioFunction
            if (
                isinstance(stdio_func, list)
                and stdio_func
                and isinstance(stdio_func[0], str)
                and stdio_func[0].startswith(priority)
            ):
                return conn

    # Fallback to the first stdio connection if no priority match
    return stdio_connections[0]


def choose_connection(server: RegistryServer) -> ConnectionDetails:
    """Chooses the most suitable connection for a server."""
    connections = server.connections  # Directly access the dataclass attribute
    if not connections:
        raise ValueError(f"No connection configurations found for server {server.qualifiedName}")

    # Prefer stdio for local servers if available
    if not server.remote:  # Directly access the dataclass attribute
        stdio_connection = choose_stdio_connection(connections)
        if stdio_connection:
            return stdio_connection

    # Otherwise, look for WebSocket (ws) connection
    ws_connection = next((conn for conn in connections if conn.type == "ws"), None)
    if ws_connection:
        return ws_connection

    # Fallback: try stdio again (even for remote, if ws wasn't found)
    stdio_connection = choose_stdio_connection(connections)
    if stdio_connection:
        return stdio_connection

    # Final fallback: return the first connection whatever it is
    return connections[0]


def normalize_server_id(server_id: str) -> str:
    """Normalizes server ID by replacing the first slash after '@' with a dash."""
    if server_id.startswith("@"):
        parts = server_id.split("/", 1)
        if len(parts) == 2:
            return f"{parts[0]}-{parts[1]}"
    return server_id


def denormalize_server_id(normalized_id: str) -> str:
    """Converts a normalized server ID back to its original form (with slash)."""
    if normalized_id.startswith("@"):
        parts = normalized_id.split("-", 1)  # Split only on the first dash
        # Check if the first part looks like a scope (@scope)
        if len(parts) == 2 and parts[0].startswith("@") and "/" not in parts[0]:
            return f"{parts[0]}/{parts[1]}"
    return normalized_id


def get_server_name(server_id: str) -> str:
    """Extracts the server name part from a potentially scoped server ID."""
    if server_id.startswith("@") and "/" in server_id:
        return server_id.split("/", 1)[1]
    return server_id


def format_run_config_values(
    connection: ConnectionDetails,
    config_values: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """Formats config values specifically for the 'run' command,
       allowing empty strings for missing required fields."""
    formatted_values: Dict[str, Any] = {}
    config_values = config_values or {}

    schema = connection.configSchema or {}
    env = schema.env if schema else {}
    if not env:
        return config_values

    required: Set[str] = set(schema.required if schema else [])

    for key, prop_details in env.items():
        value = config_values.get(key)
        prop_type = prop_details.type
        final_value: Any

        if value is not None:
            final_value = value
        elif key not in required:
            final_value = prop_details.default
        else:
            # Key is required but value is missing: use empty string
            final_value = ""

        # Handle cases where even the default might be None
        if final_value is None:
            # If required use empty string, otherwise None is acceptable for optional
            formatted_values[key] = "" if key in required else None
            continue

        try:
            formatted_values[key] = convert_value_to_type(final_value, prop_type)
        except ValueError:
            # If conversion fails, use empty string if required, None otherwise
            formatted_values[key] = "" if key in required else None

    return formatted_values

# Implementation for formatServerConfig with proxy command support


def build_dev_config(qualified_name: str,
                     user_config: Dict[str, Any],
                     api_key: Optional[str] = None,
                     config_needed: bool = True):
    """
    Formats server config for development environment.

    Creates a command that runs the Python module with the server name, config and API key,
    with platform-specific handling for Windows vs Unix-like systems.

    Args:
        qualified_name: The package identifier 
        user_config: User configuration values
        api_key: Optional API key
        config_needed: Whether config flag is needed
    """
    import sys
    import os

    # 检测是否为Windows系统
    is_windows = sys.platform == "win32"

    # 使用固定的CLI项目目录（开发环境）
    cli_dir = os.path.abspath(os.path.join(os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.dirname(__file__))))))

    if is_windows:
        # Windows使用cmd作为命令解释器
        command = "cmd"

        # 构建完整的Windows命令行
        # 使用 /c 选项指示执行完命令后退出cmd，& 为Windows命令连接符
        cmd = f"cd /d {cli_dir} & PYTHONPATH=src python -m src run {qualified_name}"

        # 添加配置（如果需要）
        if config_needed and user_config:
            # Windows中JSON字符串使用双引号包裹，需要转义
            json_config = json.dumps(user_config).replace('"', '\\"')
            cmd += f' --config "{json_config}"'

        # 添加 API 密钥（如果有提供）
        if api_key:
            cmd += f" --api-key {api_key}"

        # 构建参数列表
        args = ["/c", cmd]
    else:
        # Unix/macOS使用sh作为命令解释器
        command = "sh"

        # 构建完整的shell命令
        cmd = f"cd {cli_dir} && PYTHONPATH=src python3 -m src run {qualified_name}"

        # 添加配置（如果需要）
        if config_needed and user_config:
            # 将配置对象转为JSON字符串，并用单引号包裹（适用于shell命令）
            json_config = json.dumps(user_config)
            cmd += f" --config '{json_config}'"

        # 添加 API 密钥（如果有提供）
        if api_key:
            cmd += f" --api-key {api_key}"

        # 构建参数列表
        args = ["-c", cmd]

    return command, args


def build_test_config(qualified_name: str,
                      user_config: Dict[str, Any],
                      api_key: Optional[str] = None,
                      config_needed: bool = True,
                      package_version: str = "1.0.21"):
    """
    Formats server config for test environment.
    Uses uvx to run a specific version of mcpdock from test.pypi.org.

    Args:
        qualified_name: The package identifier
        user_config: User configuration values
        api_key: Optional API key
        config_needed: Whether config flag is needed
        package_version: The version of py-mcpdock-cli to use

    Returns:
        Tuple containing command and args
    """
    # 使用uvx从测试PyPI获取指定版本
    command = "uvx"

    # 构建参数列表
    args = [
        "--from",
        f"mcpdock=={package_version}",
        "--index-url",
        "https://test.pypi.org/simple/",
        "--extra-index-url",
        "https://pypi.org/simple/",
        '--index-strategy',
        'unsafe-best-match',
        "mcpdock",
        "run",
        qualified_name
    ]

    # 添加配置（如果需要）
    if config_needed and user_config:
        args.append("--config")
        json_config = json.dumps(user_config)
        args.append(json_config)

    # 添加 API 密钥（如果有提供）
    if api_key:
        args.append("--api-key")
        args.append(api_key)

    return command, args


def build_prod_config(qualified_name: str,
                      user_config: Dict[str, Any],
                      api_key: Optional[str] = None,
                      config_needed: bool = True):
    """
    Formats server config into a command structure with proxy command support.
    Cross-platform compatible for both Windows and Unix-like systems.

    Args:
        qualified_name: The package identifier (被代理的命令标识)
        user_config: User configuration values
        api_key: Optional API key
        config_needed: Whether config flag is needed

    Returns:
        Tuple containing command and args
    """
    import sys

    # 检测是否为Windows系统
    is_windows = sys.platform == "win32"

    # 根据系统选择合适的包管理工具命令
    if is_windows:
        # Windows下更常用npx而非uvx
        command = "npx"
    else:
        # Unix-like系统使用uvx
        command = "uvx"

    proxy_package = "@mcpspace/proxy"  # 默认代理包

    # 构建参数列表
    args = ["-y"]

    # 添加代理包标识符
    args.append(proxy_package)

    # 添加 run 命令
    args.append("run")

    # 添加实际的包标识符（被代理的命令）
    args.append(qualified_name)

    # 添加配置（如果需要）
    if config_needed and user_config:
        args.append("--config")
        # 在Windows上，可能需要特别处理JSON字符串中的引号
        json_config = json.dumps(user_config)
        if is_windows:
            json_config = json_config.replace('"', '\\"')
        args.append(json_config)

    # 添加 API 密钥（如果有提供）
    if api_key:
        args.append("--api-key")
        args.append(api_key)
    return command, args


def format_server_config(
    qualified_name: str,
    user_config: Dict[str, Any],
    api_key: Optional[str] = None,
    config_needed: bool = True,
) -> ConfiguredServer:
    command, args = None, None
    if APP_ENV == 'dev':
        command, args = build_dev_config(qualified_name, user_config, api_key, config_needed)
    elif APP_ENV == 'test':
        command, args = build_test_config(qualified_name, user_config, api_key, config_needed)
    else:
        # prod环境或其他环境
        command, args = build_prod_config(qualified_name, user_config, api_key, config_needed)
    return ConfiguredServer(
        command=command,
        args=args,
        env=user_config
    )


def build_runtime_args_and_env(connection: ConnectionDetails, config: Dict[str, Any]) -> Tuple[list, dict]:
    """
    运行阶段：根据 configSchema.args 和 config 生成最终命令行参数列表和环境变量。
    - args: 字符串直接加入，对象从 config 取值并拼接 --name=value。
    - env: 只取 config 里在 configSchema.env 里定义的 key。
    """
    schema = connection.configSchema or {}
    args_schema = schema.args or []
    env_schema = schema.env if schema else {}
    final_args = []
    for arg in args_schema:
        if isinstance(arg, str):
            final_args.append(arg)
        elif isinstance(arg, dict):
            name = arg.get("name")
            if name and name in config:
                value = config[name]
                final_args.append(f"--{name}")
                final_args.append(str(value))
    # 只保留 env_schema 里定义的 key
    final_env = {k: v for k, v in config.items() if k in env_schema}
    return final_args, final_env



from dataclasses import dataclass, field
from typing import List, Optional, Dict, Any

# Using dataclasses for structured type definitions


@dataclass
class ConfigSchemaProperty:
    # Represents a single property within the configSchema
    # Using Any for now, can be refined if specific property types are known
    type: Optional[str] = None
    description: Optional[str] = None
    default: Any = None
    required: Optional[bool] = None  # 新增，兼容 mock_servers.json
    # Add other potential schema attributes if needed


@dataclass
class ConfigSchema:
    env: Dict[str, ConfigSchemaProperty] = field(default_factory=dict)
    required: List[str] = field(default_factory=list)
    args: Optional[List[str]] = None


@dataclass
class ConnectionDetails:
    type: str
    stdioFunction: Optional[List[str]] = None
    deploymentUrl: Optional[str] = None
    published: Optional[bool] = None
    configSchema: Optional[ConfigSchema] = None  # Use the nested dataclass


@dataclass
class ConfiguredServer:
    command: str
    args: List[str]
    env: Optional[Dict[str, str]] = None


@dataclass
class MCPConfig:
    # Represents the overall configuration structure potentially saved
    mcpServers: Dict[str, ConfiguredServer] = field(default_factory=dict)


@dataclass
class ServerConfig:
    # Represents the configuration *for* a specific server instance
    # Using a general dict for extra keys corresponding to `[key: string]: unknown`
    qualifiedName: str
    connections: List[ConnectionDetails]
    remote: Optional[bool] = None
    additional_config: Dict[str, Any] = field(default_factory=dict)


@dataclass
class ClientConfig:
    # Represents the client-side configuration (e.g., installed servers)
    # Using Dict[str, Any] for flexibility as the TS type was Record<string, unknown>
    mcpServers: Dict[str, Any] = field(default_factory=dict)


@dataclass
class RegistryServer:
    # Represents a server as listed in a registry
    qualifiedName: str
    connections: List[ConnectionDetails]
    displayName: Optional[str] = None
    remote: Optional[bool] = None



"""
WebSocket Runner implementation for remote MCP servers
"""
import json
import asyncio
import signal
import sys
from typing import Dict, Any, Optional

from rich import print as rprint

from ..utils.logger import verbose


async def create_ws_runner(
    deployment_url: str,
    config: Dict[str, Any],
    api_key: Optional[str] = None
) -> None:
    """
    Creates a WebSocket runner for connecting to a remote server.
    This is a placeholder function that will be connected to a real implementation.
    """
    verbose(f"Starting WebSocket runner: {deployment_url}")
    rprint(f"[green]WebSocket connection successful: {deployment_url}[/green]")
    rprint(f"Configuration: {json.dumps(config, indent=2)}")
    rprint("Press Ctrl+C to stop the connection")

    # Setup signal handling for graceful shutdown
    def handle_sigint(sig, frame):
        rprint("[yellow]Received stop signal, closing connection...[/yellow]")
        sys.exit(0)

    signal.signal(signal.SIGINT, handle_sigint)

    # Keep the process running (equivalent to the Promise in JS)
    try:
        # This is a simple placeholder that keeps the process running
        # In a real implementation, this would be replaced with actual connection logic
        while True:
            await asyncio.sleep(1)
    except asyncio.CancelledError:
        rprint("[yellow]WebSocket connection cancelled.[/yellow]")



"""
Streamable HTTP Runner implementation for remote MCP servers (streaming HTTP transport, stdio proxy)
"""
import json
import asyncio
import signal
import sys
import time
from typing import Dict, Any, Optional, Awaitable

from rich import print as rprint
from ..utils.logger import verbose
import aiohttp

IDLE_TIMEOUT = 10 * 60  # 10分钟，单位：秒
MAX_RETRIES = 5
RETRY_DELAY = 2  # 秒，指数退避基数


async def create_stream_http_runner(
    base_url: str,
    config: Dict[str, Any],
    api_key: Optional[str] = None
) -> Awaitable[None]:
    """
    Creates a streamable HTTP runner for connecting to a remote server using stdio as proxy.
    """
    retry_count = 0
    stdin_buffer = ""
    is_ready = False
    is_shutting_down = False
    is_client_initiated_close = False
    last_activity = time.time()
    idle_check_task = None
    session = None
    post_task = None
    response_task = None
    stop_event = asyncio.Event()

    def log_with_timestamp(msg):
        verbose(f"[Runner] {msg}")

    def update_last_activity():
        nonlocal last_activity
        last_activity = time.time()

    async def cleanup():
        nonlocal is_shutting_down, is_client_initiated_close, idle_check_task, session, post_task, response_task
        if is_shutting_down:
            log_with_timestamp("Cleanup already in progress, skipping duplicate cleanup...")
            return
        log_with_timestamp("Starting cleanup process...")
        is_shutting_down = True
        is_client_initiated_close = True
        if idle_check_task:
            idle_check_task.cancel()
        if post_task:
            post_task.cancel()
        if response_task:
            response_task.cancel()
        if session:
            await session.close()
        log_with_timestamp("Cleanup completed")
        stop_event.set()

    async def handle_exit(*_):
        log_with_timestamp("Received exit signal, initiating shutdown...")
        await cleanup()
        sys.exit(0)

    def start_idle_check():
        nonlocal idle_check_task

        async def idle_checker():
            while True:
                await asyncio.sleep(60)
                idle_time = time.time() - last_activity
                if idle_time >= IDLE_TIMEOUT:
                    log_with_timestamp(f"Connection idle for {int(idle_time // 60)} minutes, initiating shutdown")
                    await handle_exit()
                    break
        idle_check_task = asyncio.create_task(idle_checker())

    async def post_stream():
        nonlocal is_ready, retry_count, session, response_task
        log_with_timestamp(f"Connecting to HTTP stream endpoint: {base_url}")
        url = base_url  # TODO: 按需拼接 config/api_key
        headers = {"Content-Type": "application/json"}
        if api_key:
            headers["Authorization"] = f"Bearer {api_key}"
        # 用队列做流式输入
        input_queue = asyncio.Queue()

        async def stdin_reader():
            nonlocal stdin_buffer
            while not is_shutting_down:
                data = await asyncio.get_event_loop().run_in_executor(None, sys.stdin.buffer.readline)
                if not data:
                    log_with_timestamp("STDIN closed (client disconnected)")
                    await handle_exit()
                    break
                update_last_activity()
                stdin_buffer += data.decode("utf-8")
                lines = stdin_buffer.split("\n")
                stdin_buffer = lines.pop() if lines else ""
                for line in [l for l in lines if l.strip()]:
                    try:
                        await input_queue.put(json.loads(line))
                    except Exception as e:
                        log_with_timestamp(f"Failed to parse stdin line: {e}")

        async def gen():
            # 先发 config
            yield json.dumps({"config": config}) + "\n"
            while not is_shutting_down:
                msg = await input_queue.get()
                yield json.dumps(msg) + "\n"
        session = aiohttp.ClientSession()
        try:
            resp = await session.post(url, data=gen(), headers=headers, timeout=None)
            is_ready = True
            log_with_timestamp("HTTP stream connection established")
            start_idle_check()
            # 启动响应流处理
            response_task = asyncio.create_task(response_stream(resp))
            # 启动stdin读取
            await stdin_reader()
        except Exception as e:
            log_with_timestamp(f"HTTP stream error: {e}")
            await handle_exit()

    async def response_stream(resp):
        try:
            async for line in resp.content:
                update_last_activity()
                try:
                    # 只处理非空行
                    s = line.decode("utf-8").strip()
                    if not s:
                        continue
                    # 允许服务端返回多行JSON
                    for l in s.split("\n"):
                        if l.strip():
                            print(l)
                except Exception as e:
                    log_with_timestamp(f"Error handling response: {e}")
        except Exception as e:
            log_with_timestamp(f"HTTP response stream error: {e}")
            await handle_exit()

    # 信号处理
    loop = asyncio.get_running_loop()
    for sig in (signal.SIGINT, signal.SIGTERM):
        try:
            loop.add_signal_handler(sig, lambda: asyncio.create_task(handle_exit()))
        except NotImplementedError:
            signal.signal(sig, lambda *_: asyncio.create_task(handle_exit()))

    # 启动主流式POST任务
    post_task = asyncio.create_task(post_stream())
    rprint(f"[green]Streamable HTTP connection established: {base_url}[/green]")
    rprint("Press Ctrl+C to stop the connection")
    await stop_event.wait()

# 用法示例：
# await create_stream_http_runner("https://example.com/stream", {"foo": "bar"}, api_key="xxx")



import subprocess
import sys
import shutil
import os
from typing import Dict, Optional, List
import questionary
from rich import print as rprint

from ..types.registry import ConnectionDetails, RegistryServer # Assuming RegistryServer includes connections
from .logger import verbose


def check_command_installed(command: str) -> bool:
    """Checks if a command is available in the system's PATH."""
    if not shutil.which(command):
        verbose(f"Command '{command}' not found in PATH.")
        return False
    # Optionally, run a version command to be more certain
    try:
        # Use shell=True if command might be an alias or shell function, but be cautious
        # Using the direct path from shutil.which is safer if possible
        cmd_path = shutil.which(command)
        if not cmd_path:
             return False # Should not happen if which found it, but safety check
        result = subprocess.run([cmd_path, "--version"], capture_output=True, text=True, timeout=5, check=False)
        verbose(f"'{command} --version' exited with code {result.returncode}")
        # Check return code - 0 usually means success
        return result.returncode == 0
    except (subprocess.TimeoutExpired, OSError, FileNotFoundError) as e:
        verbose(f"Error checking version for '{command}': {e}")
        # If version check fails, assume it might still work or is problematic
        return False # Treat version check failure as potentially not installed correctly
    except Exception as e:
        verbose(f"Unexpected error checking '{command}': {e}")
        return False

async def prompt_for_install(command: str, install_url: str, command_name: str) -> bool:
    """Asks the user if they want to install a missing command."""
    should_install = await questionary.confirm(
        f"{command_name} is required for this operation. Would you like to attempt installation?",
        default=True
    ).ask_async()

    if not should_install:
        rprint(f"[yellow]{command_name} installation declined. You can install it manually from {install_url}[/yellow]")
        return False

    return await install_command(command, install_url, command_name)

async def install_command(command: str, install_url: str, command_name: str) -> bool:
    """Attempts to install a command using provided curl/powershell scripts."""
    rprint(f"Attempting to install {command_name}...")
    try:
        if sys.platform == "win32":
            ps_command = f"powershell -ExecutionPolicy ByPass -NoProfile -Command \"irm '{install_url}/install.ps1' | iex\""
            verbose(f"Running install command (Windows): {ps_command}")
            # Using shell=True for powershell pipeline
            result = subprocess.run(ps_command, shell=True, capture_output=True, text=True, check=True)
        else:
            sh_command = f"curl -LsSf '{install_url}/install.sh' | sh"
            verbose(f"Running install command (Unix): {sh_command}")
            # Using shell=True for curl pipeline
            result = subprocess.run(sh_command, shell=True, capture_output=True, text=True, check=True)

        verbose(f"Installation stdout:\n{result.stdout}")
        if result.stderr:
             verbose(f"Installation stderr:\n{result.stderr}")
        rprint(f"[green]✓ {command_name} installed successfully.[/green]")
        return True
    except subprocess.CalledProcessError as e:
        rprint(f"[red]Failed to install {command_name}.[/red]")
        rprint(f"Stderr:\n{e.stderr}")
        rprint(f"You can try installing it manually from {install_url}")
        return False
    except Exception as e:
        rprint(f"[red]An unexpected error occurred during installation: {e}[/red]")
        rprint(f"You can try installing it manually from {install_url}")
        return False

def is_command_required(connection: ConnectionDetails, command: str) -> bool:
    """Checks if a command is listed in the stdioFunction for a stdio connection."""
    return (
        connection.type == "stdio" and
        isinstance(connection.stdioFunction, list) and
        command in connection.stdioFunction
    )

async def ensure_command_installed(
    connection: ConnectionDetails,
    command: str,
    command_name: str,
    install_url: str,
) -> bool:
    """Checks if a required command is installed, prompts to install if not."""
    if is_command_required(connection, command):
        verbose(f"{command_name} installation check required for connection type {connection.type}.")
        if not check_command_installed(command):
            rprint(f"[yellow]Required command '{command_name}' ('{command}') not found or not working.[/yellow]")
            installed = await prompt_for_install(command, install_url, command_name)
            if not installed:
                rprint(f"[yellow]Warning: {command_name} is not installed. The server might fail to launch.[/yellow]")
                return False
            # Re-check after installation attempt
            if not check_command_installed(command):
                 rprint(f"[red]Error: {command_name} installation seemed to succeed, but '{command}' is still not working correctly.[/red]")
                 return False
            return True # Installed successfully
        else:
            verbose(f"Command '{command_name}' ('{command}') is installed.")
            return True # Already installed
    else:
        verbose(f"Command '{command_name}' not required for this connection.")
        return True # Not required for this connection

async def ensure_uv_installed(connection: ConnectionDetails) -> bool:
    """Ensures UV (uvx command) is installed if required by the connection."""
    # Assuming uvx is the command to check for uv
    return await ensure_command_installed(connection, "uvx", "UV", "https://astral.sh/uv")

async def ensure_bun_installed(connection: ConnectionDetails) -> bool:
    """Ensures Bun (bunx command) is installed if required by the connection."""
    # Assuming bunx is the command to check for bun
    return await ensure_command_installed(connection, "bunx", "Bun", "https://bun.sh")

def get_runtime_environment(base_env: Optional[Dict[str, str]] = None) -> Dict[str, str]:
    """Gets runtime environment variables.

    Placeholder: This function needs to replicate the behavior of
    `@modelcontextprotocol/sdk/client/stdio.js#getDefaultEnvironment`
    if specific environment variables are required by the MCP servers.
    Currently, it merges provided base_env with the current process environment.
    """
    default_env = os.environ.copy()
    verbose("Using current process environment as default runtime environment.")
    # --- Placeholder Start ---
    # TODO: Determine if specific MCP environment variables are needed
    # and set them here if possible.
    # Example: default_env["MCP_PYTHON_CLIENT"] = "1"
    # --- Placeholder End ---

    if base_env:
        return {**default_env, **base_env}
    else:
        return default_env

def check_and_notify_remote_server(server: RegistryServer) -> bool:
    """Checks if the server is remote and prints a security notice if it is."""
    # Determine if remote based on connections or explicit flag
    is_remote = (
        any(conn.type == "ws" and conn.deploymentUrl for conn in server.connections)
        and server.remote is not False # Explicitly false overrides ws check
    )

    if is_remote:
        verbose("Remote server detected, showing security notice")
        rprint(
            "[blue]Installing remote server. Please ensure you trust the server author, "
            "especially when sharing sensitive data.[/blue]"
        )
        # Add link if available
        # rprint(f"[blue]For information on data policy, please visit: [underline]https://example.com/data-policy[/underline][/blue]")

    return is_remote


import click
import os
import sys
import json
import subprocess
from typing import Optional, Dict, Any, List

from ..utils.logger import verbose, info, error, warning
from ..config.client_config import read_config, write_config, get_config_target
from ..types.registry import ClientConfig
from ..utils.process_utils import check_server_by_package_name, remove_pid_by_server_name, is_process_running


def get_server_name(package_identifier: str) -> str:
    """
    Normalizes a package identifier to use as a key in the configuration.

    Args:
        package_identifier: The package identifier (e.g., '@org/package-name')

    Returns:
        A normalized server name to use as a key
    """
    # Handle scoped packages (@org/name format)
    if package_identifier.startswith('@') and '/' in package_identifier:
        # For scoped packages, use the part after the slash
        return package_identifier.split('/', 1)[1]
    return package_identifier


async def is_client_running(client: Optional[str] = None) -> bool:
    """
    Check if the specified client is currently running.

    Args:
        client: The client name to check

    Returns:
        Whether the client is running
    """
    if not client:
        return False

    # Map of client names to process names to check
    client_process_map = {
        "claude": "Claude",
        "vscode": "Code",
        "vscode-insiders": "Code - Insiders",
        "cline": "Cline",
        "cursor": "Cursor"
        # Add more mappings as needed
    }

    process_name = client_process_map.get(client.lower())
    if not process_name:
        verbose(f"Unknown client: {client}, can't check if it's running")
        return False

    try:
        import psutil
        for proc in psutil.process_iter(['name']):
            if process_name.lower() in proc.info['name'].lower():
                verbose(f"Client {client} is running (process {proc.info['name']})")
                return True

        verbose(f"Client {client} is not running")
        return False
    except Exception as e:
        verbose(f"Error checking if client is running: {str(e)}")
        return False


async def restart_client(client: str) -> None:
    """
    Attempt to restart the specified client application.

    Args:
        client: The client name to restart
    """
    # Map client names to the commands that can restart them
    client_restart_map = {
        "claude": "open -a Claude",
        "vscode": "code",
        "vscode-insiders": "code-insiders",
        "cline": "cline",
        "cursor": "open -a Cursor"
        # Add more mappings as needed
    }

    restart_cmd = client_restart_map.get(client.lower())
    if not restart_cmd:
        verbose(f"No restart command available for {client}")
        return

    try:
        verbose(f"Attempting to restart {client} with command: {restart_cmd}")
        subprocess.run(restart_cmd, shell=True, check=True)
        info(f"Restarted {client}")
    except subprocess.SubprocessError as e:
        error(f"Failed to restart {client}: {str(e)}")


async def prompt_for_restart(client: Optional[str] = None) -> bool:
    """
    Ask the user if they want to restart the client after configuration changes.

    Args:
        client: The client name that might need restarting

    Returns:
        True if the client was restarted, False otherwise
    """
    if not client:
        return False

    # Check if the client is currently running
    is_running = await is_client_running(client)
    if not is_running:
        verbose(f"Client {client} is not running, no need to restart")
        return False

    # In CLI, we need to prompt the user for confirmation
    # from rich.prompt import Confirm
    # should_restart = Confirm.ask(f"Would you like to restart {client} to apply changes?")
    should_restart = True
    if should_restart:
        await restart_client(client)
        return True
    else:
        info(f"Please restart {client} manually to apply changes")
        return False


async def uninstall_server(package_identifier: str, client: Optional[str] = None) -> None:
    """
    Uninstalls a server for the specified client by removing it from the client's configuration.

    Args:
        package_identifier: The server package to uninstall
        client: The client to uninstall the server from
    """
    try:
        client_name = client or "claude"  # Default to Claude if no client specified
        verbose(f"Uninstalling {package_identifier} from {client_name}")

        # Get configuration target to determine how to handle the uninstallation
        target = get_config_target(client_name)
        if target["type"] == "command":
            info(f"Uninstallation is currently not supported for {client_name}")
            return

        # Read current configuration
        verbose(f"Reading configuration for client: {client_name}")
        client_config = read_config(client_name)

        # Get normalized server name to use as key
        server_name = get_server_name(package_identifier)
        verbose(f"Normalized server ID: {server_name}")

        # Check if server exists in config
        if not hasattr(client_config, 'mcpServers') or not isinstance(client_config.mcpServers, dict):
            verbose(f"Client {client_name} has no MCP servers configured")
            info(f"Server {package_identifier} is not installed for {client_name}")
            return

        if server_name not in client_config.mcpServers:
            verbose(f"Server {server_name} not found in configuration for {client_name}")
            info(f"Server {package_identifier} is not installed for {client_name}")
            return

        # Remove the server from configuration
        verbose(f"Removing server {server_name} from configuration")
        del client_config.mcpServers[server_name]

        # Write updated configuration
        verbose("Writing updated configuration...")
        write_config(client_config, client_name)

        # Check if the server process is running and clean up PID records
        is_running, pid = check_server_by_package_name(package_identifier)
        if is_running and pid:
            verbose(f"Found running server process with PID {pid}")
            # Remove from PID file but don't kill the process automatically
            remove_pid_by_server_name(server_name, client_name)
            info(f"Note: The server process (PID: {pid}) is still running. You may want to restart your client.")

        info(f"Successfully uninstalled {package_identifier} from {client_name}")

        # Prompt user to restart client to apply changes
        await prompt_for_restart(client_name)

    except Exception as e:
        error(f"Failed to uninstall {package_identifier}: {str(e)}")
        # Structured error output for upstream processes
        error_obj = {
            "error_type": type(e).__name__,
            "message": str(e),
            "traceback": __import__('traceback').format_exc()
        }
        print(json.dumps(error_obj, ensure_ascii=False), file=sys.stderr)
        return 1


@click.command("uninstall")
@click.argument("mcp_server", type=click.STRING)
@click.option("--client", type=click.STRING, help="Specify the target AI client")
def uninstall(mcp_server: str, client: Optional[str]):
    """Uninstall an AI mcp-server from the specified client."""
    verbose(f"Attempting to uninstall {mcp_server}...")

    try:
        # Run the async uninstallation process
        import asyncio
        asyncio.run(uninstall_server(mcp_server, client))
    except Exception as e:
        # Output structured JSON error information for upstream processes
        import sys
        import traceback
        error_obj = {
            "error_type": type(e).__name__,
            "message": str(e),
            "traceback": traceback.format_exc()
        }
        print(json.dumps(error_obj, ensure_ascii=False), file=sys.stderr)
        return 1



import click
from .commands.install import install
from .commands.run import run
from .commands.query_pid import query_pid
from .commands.uninstall import uninstall


@click.group()
@click.version_option(version="1.0.0", package_name="py-cli")  # Corresponds to program.version("1.0.0")
def cli():
    """A custom CLI tool translated to Python."""
    pass


cli.add_command(install)
cli.add_command(run)
cli.add_command(query_pid)
cli.add_command(uninstall)



"""
进程管理工具模块

该模块提供了与进程管理相关的工具函数，包括：
- 查找进程ID
- 保存PID文件
- 读取PID文件
"""
import json
import os
import time
import socket
import psutil
import sys
import tempfile
from filelock import FileLock
from typing import List, Dict, Any, Optional, Tuple
from logging import error
from ..utils.logger import verbose


def find_server_process_by_command(command: str, args: List[str]) -> Optional[int]:
    """
    根据命令和参数查找匹配的进程并返回其PID
    针对Python CLI启动的MCP服务器进程进行了特殊处理

    Args:
        command: 命令名称 (如 'python' 或 'npx')
        args: 命令参数 (如 ['-m', 'server_name'] 或 ['-y', '@package/name'])

    Returns:
        匹配进程的PID，如果没找到则返回None
    """
    try:
        command_str = ' '.join([command] + args)
        verbose(f"[Runner] 查找进程，命令: {command_str}")

        import sys

        # 检测是否为Windows系统
        is_windows = sys.platform == "win32"

        # 提取关键标识符 - 对于MCP服务器尤其重要
        # 通常是最后一个参数或包含@的参数
        key_identifiers = []
        server_name = None

        # 查找包含 @ 的参数，通常是包名或服务器标识
        for arg in args:
            if '@' in arg:
                key_identifiers.append(arg)
                # 保存可能的服务器名称
                if '/' in arg:
                    # 对于 @scope/package 格式，提取 package 部分
                    server_name = arg.split('/', 1)[1] if arg.startswith('@') else arg

        # 如果没有找到包含 @ 的参数，使用最后一个参数
        if not key_identifiers and args:
            key_identifiers.append(args[-1])
            server_name = args[-1]

        # 加入命令本身作为标识符
        if command not in ['python', 'python3', 'node', 'npx', 'uvx', 'npm']:
            key_identifiers.append(command)

        verbose(f"[Runner] 使用关键标识符: {key_identifiers}")

        # 1. 首先尝试直接匹配完整命令行
        for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
            try:
                if proc.info['cmdline']:
                    proc_cmd = ' '.join(proc.info['cmdline'])
                    # 检查命令是否完全匹配
                    if command_str in proc_cmd:
                        verbose(f"[Runner] 找到完全匹配进程 PID: {proc.info['pid']}, 命令行: {proc_cmd}")
                        return proc.info['pid']
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass

        # 2. 处理NPX和Node特殊情况
        if command.lower() == 'npx':
            # 通常npx会启动node进程
            node_patterns = ["node", "node.exe"]
            if len(args) >= 2 and args[0] == '-y':
                # 如果是npx -y package的格式，实际执行的可能是node命令
                package_name = args[1]
                verbose(f"[Runner] NPX命令检测：寻找包含 '{package_name}' 的Node进程")

                for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
                    try:
                        if proc.info['cmdline']:
                            cmdline = proc.info['cmdline']
                            cmd_str = ' '.join(cmdline)

                            # 检查是否是node进程并且命令行包含包名
                            proc_name = os.path.basename(cmdline[0].lower())
                            if (any(pattern in proc_name for pattern in node_patterns) and
                                    package_name in cmd_str):
                                verbose(f"[Runner] 找到匹配的NPX启动的进程 PID: {proc.info['pid']}, 命令行: {cmd_str}")
                                return proc.info['pid']
                    except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                        pass

        # 3. 查找Python CLI启动的子进程
        parent_processes = []
        child_processes = []

        # 特别检查可能是Python/Node启动的进程
        potential_parent_commands = ['python', 'python3', 'node', 'node.exe']

        # 查找所有潜在的父进程和子进程
        for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
            try:
                if proc.info['cmdline']:
                    cmdline = proc.info['cmdline']
                    proc_cmd = ' '.join(cmdline)

                    # 检查是否是可能的父进程
                    proc_name = os.path.basename(cmdline[0].lower())
                    if any(parent_cmd in proc_name for parent_cmd in potential_parent_commands):
                        # 如果命令行包含任何关键标识符，这可能是我们要找的进程
                        if any(identifier in proc_cmd for identifier in key_identifiers):
                            parent_processes.append((proc.info['pid'], proc_cmd))

                    # 记录所有可能的子进程
                    if server_name and server_name.lower() in proc_cmd.lower():
                        child_processes.append((proc.info['pid'], proc_cmd))
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass

        # 4. 如果找到了可能的父进程，返回最匹配的一个
        if parent_processes:
            # 按照命令行长度排序，更长的命令行通常包含更多信息，匹配度更高
            parent_processes.sort(key=lambda x: len(x[1]), reverse=True)
            verbose(f"[Runner] 找到最匹配的父进程 PID: {parent_processes[0][0]}, 命令行: {parent_processes[0][1]}")
            return parent_processes[0][0]

        # 5. 如果没有找到父进程，但找到了可能的子进程，返回第一个
        if child_processes:
            verbose(f"[Runner] 找到可能的服务器进程 PID: {child_processes[0][0]}, 命令行: {child_processes[0][1]}")
            return child_processes[0][0]

        verbose("[Runner] 未找到匹配的服务器进程")
        return None
    except Exception as e:
        error(f"[Runner] 查找进程异常: {str(e)}")
        return None


def save_pid_to_file(server_pid: int, server_name: str, client_name: str, command: str, args: List[str]) -> Optional[str]:
    """
    将进程ID保存到PID文件，记录MCP服务器的使用情况，包括客户端名称。
    同时查找并保存与该MCP服务器相关的进程ID（包括父进程和子进程）。

    Args:
        server_pid: 服务器进程ID
        server_name: MCP服务器名称
        client_name: 客户端名称
        command: 启动命令
        args: 命令参数

    Returns:
        成功时返回PID文件路径，失败时返回None
    """
    # 获取PID文件路径
    current_dir = os.path.dirname(os.path.abspath(__file__))
    base_dir = os.path.dirname(os.path.dirname(current_dir))
    pid_file = os.path.join(base_dir, "pids", "run-mcp-srv.pid")
    lock_file = f"{pid_file}.lock"

    try:
        # 获取当前CLI进程的PID
        cli_pid = os.getpid()
        verbose(f"[Runner] 当前CLI进程PID: {cli_pid}")

        # 获取客户端进程ID（可能是第三级父进程）
        client_pid = None
        try:
            # 尝试找到三级父进程（如果存在）
            try:
                process = psutil.Process(server_pid)
                parent = process.parent()
                if parent:
                    grandparent = parent.parent()
                    if grandparent:
                        # 找到三级父进程
                        client_pid = grandparent.pid
                        if client_pid:
                            verbose(f"[Runner] 找到MCP服务器的三级父进程 PID: {client_pid}")
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                verbose(f"[Runner] 无法获取进程 {server_pid} 的父进程")

            verbose(f"[Runner] 为MCP服务器 {server_name} 找到客户端进程ID: {client_pid}")
        except Exception as e:
            verbose(f"[Runner] 查找客户端进程时出错: {str(e)}")

        with FileLock(lock_file, timeout=5):
            pid_data = {}

            # 如果文件存在，读取现有数据
            if os.path.exists(pid_file):
                with open(pid_file, 'r') as f:
                    content = f.read().strip()
                    if not content:
                        verbose(f"[Runner] PID文件为空: {pid_file}")
                        content = '{}'
                    pid_data = json.loads(content)

            # 如果当前服务器名称不存在，初始化为数组
            if server_name not in pid_data:
                pid_data[server_name] = []

            # 客户端信息
            client_info = {
                "client_name": client_name,
                "server_id": server_pid,  # mcp server 服务器进程ID
                "client_pid": client_pid,  # 客户端进程ID（第三级父进程）
                "cli_pid": cli_pid,       # 当前CLI进程的PID
                "command": command,
                "args": args,
                "timestamp": int(time.time()),
                "hostname": os.uname().nodename if hasattr(os, 'uname') else socket.gethostname()
            }

            # 检查是否已存在相同客户端，如果存在则更新
            found = False
            for i, entry in enumerate(pid_data[server_name]):
                if entry.get("client_name") == client_name:
                    pid_data[server_name][i] = client_info
                    found = True
                    break

            # 如果不存在，则添加到列表
            if not found:
                pid_data[server_name].append(client_info)

            # 写回文件
            with open(pid_file, 'w') as f:

                json.dump(pid_data, f, indent=2)

            verbose(f"[Runner] PID信息已保存到文件: {pid_file}")
            return pid_file

    except Exception as e:
        error(f"[Runner] 保存PID文件失败: {str(e)}")
        return None


def read_pid_file() -> Optional[Dict[str, Any]]:
    """
    读取PID文件，按优先级尝试查找并读取PID文件
    使用文件锁防止与并发写入操作的冲突

    Returns:
        成功时返回PID信息字典，失败时返回None
    """
    # 按照写入文件时的优先级顺序尝试查找
    # 1. 项目内部日志目录
    try:
        # 获取PID文件路径
        current_dir = os.path.dirname(os.path.abspath(__file__))
        base_dir = os.path.dirname(os.path.dirname(current_dir))
        pid_file = os.path.join(base_dir, "pids", "run-mcp-srv.pid")
        lock_file = f"{pid_file}.lock"

        if os.path.exists(pid_file):
            # 使用文件锁防止与并发写入操作冲突
            with FileLock(lock_file, timeout=2):
                with open(pid_file, 'r') as f:
                    pid_info = json.load(f)
                    verbose(f"[Runner] 从项目日志目录读取到PID信息: {pid_info}")
                    return pid_info
    except Exception as e:
        verbose(f"[Runner] 读取项目日志目录PID文件失败: {str(e)}")

    # 2. 用户主目录
    try:
        home_dir = os.path.expanduser("~")
        pid_file = os.path.join(home_dir, ".mcpdock", "run-mcp-srv.pid")
        lock_file = f"{pid_file}.lock"

        if os.path.exists(pid_file):
            # 使用文件锁防止与并发写入操作冲突
            with FileLock(lock_file, timeout=2):
                with open(pid_file, 'r') as f:
                    pid_info = json.load(f)
                    verbose(f"[Runner] 从用户主目录读取到PID信息: {pid_info}")
                    return pid_info
    except Exception as e:
        verbose(f"[Runner] 读取用户主目录PID文件失败: {str(e)}")

    # 3. 系统临时目录
    try:
        temp_dir = tempfile.gettempdir()
        pid_file = os.path.join(temp_dir, "mcpdock-srv.pid")
        lock_file = f"{pid_file}.lock"

        if os.path.exists(pid_file):
            # 使用文件锁防止与并发写入操作冲突
            with FileLock(lock_file, timeout=2):
                with open(pid_file, 'r') as f:
                    pid_info = json.load(f)
                    verbose(f"[Runner] 从临时目录读取到PID信息: {pid_info}")
                    return pid_info
    except Exception as e:
        verbose(f"[Runner] 读取临时目录PID文件失败: {str(e)}")

    verbose("[Runner] 在所有位置均未找到有效的PID文件")
    return None


def is_process_running(pid: int) -> bool:
    """
    检查指定PID的进程是否正在运行

    Args:
        pid: 进程ID

    Returns:
        进程是否存在且正在运行
    """
    try:
        # 检查进程是否存在
        process = psutil.Process(pid)
        # 进一步检查进程状态
        return process.is_running() and process.status() != psutil.STATUS_ZOMBIE
    except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
        return False
    except Exception as e:
        error(f"[Runner] 检查进程状态异常: {str(e)}")
        return False


def check_server_process() -> Tuple[bool, Optional[Dict[str, Any]]]:
    """
    检查MCP服务器进程是否正在运行

    Returns:
        (是否正在运行, PID信息)
    """
    # 尝试读取PID文件
    pid_info = read_pid_file()
    if not pid_info:
        return False, None

    # 获取PID - 兼容新旧格式：先尝试读取server_id，如果不存在则使用pid字段
    server_id = pid_info.get("server_id")
    if not server_id:
        server_id = pid_info.get("pid")  # 兼容旧格式

    if not server_id:
        return False, pid_info

    # 检查进程是否在运行
    is_running = is_process_running(server_id)
    return is_running, pid_info


def check_server_by_package_name(package_name: str) -> Tuple[bool, Optional[int]]:
    """
    通过包名检查MCP服务器是否正在运行

    这个方法不依赖PID文件，而是直接搜索运行中的进程，查找包含指定包名的进程

    Args:
        package_name: 服务器包名，如 '@suekou/mcp-notion-server'

    Returns:
        (是否正在运行, 进程ID如果找到)
    """
    try:
        verbose(f"[Runner] 通过包名搜索服务器进程: {package_name}")

        # 处理包名格式
        server_id = package_name
        server_name = None

        # 处理 @scope/package 格式
        if '/' in package_name and package_name.startswith('@'):
            # 提取包名的主要部分（不含scope）
            server_name = package_name.split('/', 1)[1]
        else:
            server_name = package_name

        # 去除特殊字符，便于匹配
        clean_name = server_name.replace('-', '').lower() if server_name else ''

        # 潜在进程类型
        potential_cmd_patterns = ['node', 'python', 'python3']
        matching_processes = []

        # 搜索所有匹配的进程
        for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
            try:
                if proc.info['cmdline']:
                    cmdline = proc.info['cmdline']
                    cmd_str = ' '.join(cmdline).lower()

                    # 检查命令行是否包含包名
                    if package_name.lower() in cmd_str:
                        matching_processes.append((proc.info['pid'], cmd_str, 3))  # 完全匹配权重最高
                    elif server_name and server_name.lower() in cmd_str:
                        matching_processes.append((proc.info['pid'], cmd_str, 2))  # 不含scope的包名匹配
                    elif clean_name and clean_name in cmd_str.replace('-', ''):
                        matching_processes.append((proc.info['pid'], cmd_str, 1))  # 去除横线后的匹配

                    # 特殊检测: 看命令行是否有可能是MCP服务器
                    proc_base = os.path.basename(cmdline[0]).lower()
                    if any(pattern in proc_base for pattern in potential_cmd_patterns):
                        if 'mcp' in cmd_str and (server_name in cmd_str or clean_name in cmd_str.replace('-', '')):
                            matching_processes.append((proc.info['pid'], cmd_str, 2))  # MCP相关进程

            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass

        # 按照匹配权重和命令行长度排序
        if matching_processes:
            # 先按权重排序，权重相同时按命令行长度排序
            matching_processes.sort(key=lambda x: (x[2], len(x[1])), reverse=True)
            best_match = matching_processes[0]
            verbose(f"[Runner] 找到匹配的服务器进程 PID: {best_match[0]}, 命令行: {best_match[1]}")
            return True, best_match[0]

        verbose(f"[Runner] 未找到包含 '{package_name}' 的服务器进程")
        return False, None
    except Exception as e:
        error(f"[Runner] 通过包名查找进程异常: {str(e)}")
        return False, None


def find_child_processes(parent_pid: int, server_id: Optional[str] = None) -> List[Tuple[int, str]]:
    """
    查找指定父进程的所有子进程

    Args:
        parent_pid: 父进程ID
        server_id: 可选的服务器标识符，用于过滤子进程

    Returns:
        匹配的子进程列表，每项包含(pid, 命令行)
    """
    try:
        matching_children = []

        # 尝试获取父进程
        try:
            parent = psutil.Process(parent_pid)
        except (psutil.NoSuchProcess, psutil.AccessDenied):
            verbose(f"[Runner] 无法访问父进程 PID: {parent_pid}")
            return matching_children

        # 获取所有子进程
        children = parent.children(recursive=True)

        # 如果没有指定服务器ID，返回所有子进程
        if not server_id:
            return [(child.pid, ' '.join(child.cmdline())) for child in children if child.is_running()]

        # 处理服务器标识符以便匹配
        server_name = None
        clean_name = None

        # 处理 @scope/package 格式
        if server_id and '/' in server_id and server_id.startswith('@'):
            server_name = server_id.split('/', 1)[1]
        else:
            server_name = server_id

        # 去除特殊字符以便更灵活匹配
        if server_name:
            clean_name = server_name.replace('-', '').lower()

        # 过滤匹配的子进程
        for child in children:
            try:
                if not child.is_running():
                    continue

                cmd_str = ' '.join(child.cmdline()).lower()

                # 使用多种匹配策略
                if server_id and server_id.lower() in cmd_str:
                    matching_children.append((child.pid, cmd_str))
                elif server_name and server_name.lower() in cmd_str:
                    matching_children.append((child.pid, cmd_str))
                elif clean_name and clean_name in cmd_str.replace('-', ''):
                    matching_children.append((child.pid, cmd_str))
                elif 'mcp' in cmd_str and (server_name in cmd_str or
                                           (clean_name and clean_name in cmd_str.replace('-', ''))):
                    matching_children.append((child.pid, cmd_str))
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                # 子进程可能已经结束
                continue

        # 按命令行长度排序，通常更长的命令行包含更多信息
        matching_children.sort(key=lambda x: len(x[1]), reverse=True)

        if matching_children:
            verbose(f"[Runner] 找到 {len(matching_children)} 个匹配的子进程，第一个: "
                    f"PID: {matching_children[0][0]}, 命令行: {matching_children[0][1]}")
        else:
            verbose(f"[Runner] 未找到父进程 {parent_pid} 下匹配的子进程")

        return matching_children
    except Exception as e:
        error(f"[Runner] 查找子进程异常: {str(e)}")
        return []


def find_server_process_from_current() -> Tuple[bool, Optional[int], Optional[str]]:
    """
    从当前进程开始，查找MCP服务器进程

    利用进程的父子关系，查找当前CLI工具启动的MCP服务器进程

    Returns:
        (是否找到, 进程ID, 命令行)
    """
    try:
        # 获取当前进程ID
        current_pid = os.getpid()
        verbose(f"[Runner] 当前进程ID: {current_pid}")

        # 查找当前进程启动的所有子进程
        child_processes = find_child_processes(current_pid)

        if not child_processes:
            verbose("[Runner] 当前进程没有子进程")
            return False, None, None

        # 过滤可能是MCP服务器的子进程
        mcp_candidates = []

        for pid, cmd_str in child_processes:
            cmd_lower = cmd_str.lower()
            # 查找命令行中包含MCP相关关键词的进程
            if any(kw in cmd_lower for kw in ['mcp', 'notion-server', 'node', 'python']):
                mcp_candidates.append((pid, cmd_str))

        if mcp_candidates:
            # 按命令行长度排序
            mcp_candidates.sort(key=lambda x: len(x[1]), reverse=True)
            best_match = mcp_candidates[0]
            verbose(f"[Runner] 找到最可能的MCP服务器进程: PID: {best_match[0]}, 命令行: {best_match[1]}")
            return True, best_match[0], best_match[1]

        verbose("[Runner] 未找到可能的MCP服务器子进程")
        return False, None, None
    except Exception as e:
        error(f"[Runner] 从当前进程查找服务器异常: {str(e)}")
        return False, None, None


def remove_pid_by_server_name(server_name: str, client_name: Optional[str] = None, target_pid: Optional[int] = None) -> None:
    """
    根据服务器名称或目标进程ID从PID文件中移除对应的PID信息。
    支持通过当前CLI进程ID匹配删除记录。

    Args:
        server_name: 服务器名称
        client_name: 客户端名称（如果提供，只移除该客户端的记录；否则移除服务器的所有记录）
        target_pid: 目标进程ID（如果提供，会匹配server_id、client_pid和cli_pid）

    Returns:
        None
    """
    try:
        # 获取当前CLI进程ID
        current_cli_pid = os.getpid()
        verbose(f"[Runner] 当前CLI进程PID: {current_cli_pid}")

        # 获取PID文件路径
        current_dir = os.path.dirname(os.path.abspath(__file__))
        base_dir = os.path.dirname(os.path.dirname(current_dir))
        pid_file = os.path.join(base_dir, "pids", "run-mcp-srv.pid")
        lock_file = f"{pid_file}.lock"

        # 使用文件锁防止并发写入
        with FileLock(lock_file, timeout=5):
            if not os.path.exists(pid_file):
                verbose(f"[Runner] PID文件不存在: {pid_file}")
                return

            with open(pid_file, 'r') as f:
                content = f.read().strip()
                if not content:
                    verbose(f"[Runner] PID文件为空: {pid_file}")
                    return
                pid_data = json.loads(content)

            # 检查服务器名称是否存在
            if server_name not in pid_data:
                verbose(f"[Runner] 服务器名称 {server_name} 不存在于PID文件中")
                return

            entries_to_remove = []

            # 优先使用当前CLI进程ID匹配记录
            for i, entry in enumerate(pid_data[server_name]):
                cli_pid = entry.get("cli_pid")

                # 如果当前CLI进程ID与记录中的CLI PID匹配，标记为删除
                if cli_pid == current_cli_pid:
                    entries_to_remove.append(i)
                    verbose(f"[Runner] 通过当前CLI进程ID {current_cli_pid} 找到匹配记录，客户端: {entry.get('client_name')}")

            # 如果没有通过CLI PID找到记录且提供了目标进程ID，查找所有包含该PID的记录
            if not entries_to_remove and target_pid:
                for i, entry in enumerate(pid_data[server_name]):
                    server_id = entry.get("server_id")
                    client_pid = entry.get("client_pid")
                    cli_pid = entry.get("cli_pid")

                    # 检查目标PID是否匹配服务器ID、客户端进程ID或CLI进程ID
                    if server_id == target_pid or client_pid == target_pid or cli_pid == target_pid:
                        entries_to_remove.append(i)
                        verbose(f"[Runner] 已找到匹配目标PID {target_pid} 的记录，客户端: {entry.get('client_name')}")

            # 如果通过PID未找到记录且提供了客户端名称，只移除该客户端的记录
            elif not entries_to_remove and client_name:
                for i, entry in enumerate(pid_data[server_name]):
                    if entry.get("client_name") == client_name:
                        entries_to_remove.append(i)
                        verbose(f"[Runner] 已找到客户端 {client_name} 的记录")
                        break

            # 如果既没有通过PID找到记录也没有指定客户端名称，移除服务器的所有记录
            elif not entries_to_remove and not target_pid and not client_name:
                verbose(f"[Runner] 移除服务器 {server_name} 的所有记录")
                del pid_data[server_name]

            # 按索引从高到低删除，避免索引变化导致的问题
            if entries_to_remove:
                entries_to_remove.sort(reverse=True)
                for i in entries_to_remove:
                    removed_entry = pid_data[server_name].pop(i)
                    verbose(f"[Runner] 已从文件中移除记录，客户端: {removed_entry.get('client_name')}, "
                            f"服务器ID: {removed_entry.get('server_id')}, 客户端进程ID: {removed_entry.get('client_pid')}, "
                            f"CLI进程ID: {removed_entry.get('cli_pid')}")

                # 如果该服务器没有客户端了，移除服务器记录
                if not pid_data[server_name]:
                    del pid_data[server_name]
                    verbose(f"[Runner] 服务器 {server_name} 没有客户端记录，已移除")

            # 如果文件中没有服务器了，删除文件
            if not pid_data:
                os.remove(pid_file)
                verbose(f"[Runner] PID文件已被删除: {pid_file}")
            else:
                # 否则更新文件内容
                with open(pid_file, 'w') as f:
                    json.dump(pid_data, f, indent=2)
                verbose(f"[Runner] PID文件已更新")
    except Exception as e:
        error(f"[Runner] 移除PID记录失败: {str(e)}")



"""
Runners for different types of MCP servers.

This package contains implementations for running different types of MCP servers:
- stdio_runner: For running local servers via stdio communication
- ws_runner: For connecting to remote WebSocket servers
- command_runner: For running Python/Node packages using uv/npx
"""
from .stdio_runner import create_stdio_runner
from .ws_runner import create_ws_runner
from .stream_http_runner import create_stream_http_runner
from .command_runner import create_uv_runner, create_npx_runner, run_package_with_command

__all__ = [
    'create_stdio_runner',
    'create_ws_runner',
    'create_uv_runner',
    'create_npx_runner',
    'create_stream_http_runner',
    'run_package_with_command'
]



"""
Registry module for MCP servers.

This module handles resolving package names to server metadata and fetching configuration.
"""
from typing import Dict, Any, Tuple, Optional
import json
import os
import aiohttp
from cli.config.app_config import API_BASE  # 添加 HTTP 客户端库

from .utils.logger import verbose
from .types.registry import (
    RegistryServer, ConnectionDetails, ConfigSchema, ConfigSchemaProperty
)


MOCK_SERVERS_PATH = os.path.join(os.path.dirname(__file__), "mock_servers.json")


def _load_mock_servers():
    with open(MOCK_SERVERS_PATH, "r", encoding="utf-8") as f:
        return json.load(f)


def _dict_to_registry_server(data):
    # Helper to recursively convert dict to dataclasses
    def _to_config_schema_property(prop):
        return ConfigSchemaProperty(**prop)

    def _to_config_schema(schema):
        env = {k: _to_config_schema_property(v) for k, v in schema.get("env", {}).items()}
        return ConfigSchema(env=env, required=schema.get("required", []), args=schema.get("args"))

    def _to_connection_details(conn):
        config_schema = None
        if "configSchema" in conn and conn["configSchema"]:
            config_schema = _to_config_schema(conn["configSchema"])
        return ConnectionDetails(
            type=conn["type"],
            stdioFunction=conn.get("stdioFunction"),
            deploymentUrl=conn.get("deploymentUrl"),
            published=conn.get("published"),
            configSchema=config_schema
        )
    return RegistryServer(
        qualifiedName=data["qualifiedName"],
        displayName=data["displayName"],
        remote=data["remote"],
        connections=[_to_connection_details(c) for c in data["connections"]]
    )


mock_servers_data = _load_mock_servers()


# 定义 API 基础 URL
# API_BASE_URL = "http://your-php-api-url"  # 替换为您实际的 PHP API URL
API_BASE_URL = f"{API_BASE}/api/v1"


async def resolve_package(package_name: str) -> RegistryServer:
    """
    Resolves a package name to server metadata by calling the PHP API.

    Args:
        package_name: The qualified name of the package to resolve

    Returns:
        RegistryServer object with the server metadata
    """
    verbose(f"Resolving package {package_name} via API")

    try:
        async with aiohttp.ClientSession() as session:
            # 调用 PHP 的 getByQualifiedName API
            async with session.get(f"{API_BASE_URL}/mcp-servers/{package_name}") as response:
                if response.status == 200:
                    # 获取响应数据 - PHP API 直接返回原始数据，不被 data 字段包裹
                    server_data = await response.json()

                    # 将 API 响应数据转换为 RegistryServer 对象
                    return _dict_to_registry_server(server_data)
                else:
                    verbose(f"API error: {response.status} - {await response.text()}")

                    # 如果 API 调用失败，可以选择回退到本地 mock 数据
                    verbose("Falling back to mock data")
                    # Find server by qualifiedName in the array
                    data = next(
                        (server for server in mock_servers_data if server["qualifiedName"] == package_name), None)
                    if not data:
                        # Fallback to default if available
                        data = next(
                            (server for server in mock_servers_data if server["qualifiedName"] == "default"), None)
                    if not data:
                        # If still not found and we have at least one server, use the first one as fallback
                        data = mock_servers_data[0] if mock_servers_data else None

                    return _dict_to_registry_server(data) if data else None
    except Exception as e:
        verbose(f"Error calling API: {str(e)}")
        # 发生异常时回退到 mock 数据
        verbose("Falling back to mock data due to exception")
        data = next((server for server in mock_servers_data if server["qualifiedName"] == package_name), None)
        if not data:
            data = next((server for server in mock_servers_data if server["qualifiedName"] == "default"), None)
        if not data:
            data = mock_servers_data[0] if mock_servers_data else None

        return _dict_to_registry_server(data) if data else None


async def fetch_connection(
    package_name: str,
    config: Dict[str, Any]
) -> Dict[str, Any]:
    """
    Fetches server connection details for a package with given configuration.

    This function uses the package name to resolve the server details and then
    formats the connection configuration based on the schema and user provided config.

    Args:
        package_name: The name of the package to fetch connection details for
        config: User-provided configuration values

    Returns:
        Dictionary containing command, args and env configuration for the connection
    """
    verbose(f"Fetching connection details for {package_name} with config: {config}")

    # Get server metadata first
    server_data = await resolve_package(package_name)

    # Find stdio connection
    stdio_connection = next((conn for conn in server_data.connections if conn.type == "stdio"), None)
    if not stdio_connection:
        verbose("No stdio connection found in server metadata")
        return {}

    # Get command type (npx, uv, etc.) from stdioFunction
    command = "python"  # Default as fallback
    if stdio_connection.stdioFunction and len(stdio_connection.stdioFunction) > 0:
        command = stdio_connection.stdioFunction[0]

    # Get args from the connection schema
    args = []
    if stdio_connection.configSchema and stdio_connection.configSchema.args:
        args = stdio_connection.configSchema.args
    else:
        # Fallback default
        args = ["-m", package_name]

    # Return formatted connection config
    connection_config = {
        "command": command,
        "args": args,
        "env": config  # User-provided environment variables
    }

    verbose(f"Formatted connection config: {connection_config}")
    return connection_config


async def fetch_config_with_api_key(
    package_name: str,
    api_key: str
) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    """
    Fetches server and configuration information using an API key.

    Args:
        package_name: The name of the package to fetch configuration for
        api_key: The API key to authenticate with

    Returns:
        A tuple containing (server_info, config_info)
    """
    verbose(f"Mock fetching config for {package_name} using API key")

    # This is a placeholder implementation
    # In a real implementation, this would make an API call
    # to retrieve the server and configuration information

    return {}, {}  # Empty dicts for now as placeholder



"""
STDIO Runner implementation for local MCP servers

This module provides functionality for creating and managing a connection with an MCP server
through standard input/output pipes.
"""
import json
import asyncio
from logging import error
import mcp.types as types
import signal
import sys
import os
import anyio
import threading
from typing import Dict, Any, Optional, Awaitable
from contextlib import AsyncExitStack

from ..utils.logger import verbose
from ..types.registry import RegistryServer
from ..utils.runtime import get_runtime_environment
from mcp import StdioServerParameters, ClientSession
from mcp.client.stdio import stdio_client

# 导入工具模块
from ..utils.process_utils import find_server_process_by_command, save_pid_to_file
from ..utils.mcp_utils import (
    process_client_request,
    handle_single_server_message,
    initialize_session
)


async def create_stdio_runner(
    server_details: RegistryServer,
    config: Dict[str, Any],
    api_key: Optional[str] = None,
    analytics_enabled: bool = False
) -> Awaitable[None]:
    """创建并运行 STDIO 代理服务器"""
    verbose(f"Starting STDIO proxy runner: {server_details.qualifiedName}")
    is_shutting_down = False
    exit_stack = AsyncExitStack()

    def handle_error(error: Exception, context: str) -> Exception:
        verbose(f"[Runner] {context}: {error}")
        return error

    async def cleanup() -> None:
        nonlocal is_shutting_down
        if is_shutting_down:
            verbose("[Runner] Cleanup already in progress, skipping...")
            return
        verbose("[Runner] Starting cleanup...")
        is_shutting_down = True
        try:
            # Remove PID by server name and client name
            from ..utils.process_utils import remove_pid_by_server_name

            # 尝试获取第三级进程作为客户端标识 - 与保存时使用相同的逻辑
            client_name = "unknown_client"
            try:
                import psutil
                current_process = psutil.Process()

                # 收集完整的进程链
                process_chain = []
                process = current_process
                while process:
                    process_chain.append(process.name())
                    process = process.parent()

                # 打印完整进程链用于调试
                verbose(f"[Runner] 清理时的完整进程链: {' -> '.join(reversed(process_chain))}")

                # 尝试获取第四级进程（如果存在）
                if len(process_chain) >= 3:
                    client_name = process_chain[2]
                    verbose(f"[Runner] 清理时使用第四级进程作为客户端: {client_name}")
                else:
                    # 如果进程链不够长，使用最后一个进程（顶层进程）
                    client_name = process_chain[-1] if process_chain else "unknown_client"
                    verbose(f"[Runner] 清理时进程链不够长，使用顶层进程作为客户端: {client_name}")
            except Exception as e:
                verbose(f"[Runner] 清理时获取客户端进程名称失败: {e}，使用默认值: {client_name}")

            remove_pid_by_server_name(server_details.qualifiedName, client_name)
            verbose(f"[Runner] 已从PID文件中移除服务器 '{server_details.qualifiedName}' 的客户端 '{client_name}' 记录")

            await exit_stack.aclose()
            verbose("[Runner] Resources closed successfully")
        except Exception as error:
            handle_error(error, "Error during cleanup")
        verbose("[Runner] Cleanup completed")

    async def handle_sigint():
        verbose("[Runner] Received interrupt signal, shutting down...")
        await cleanup()
        # 立即打印一条确认消息，让用户知道CTRL+C已被捕获
        verbose("\n[CTRL+C] 正在关闭服务，请稍候...")
        # 可选：设置一个短暂的超时，然后强制退出
        import threading
        threading.Timer(2.0, lambda: os._exit(0)).start()

    # 获取连接配置
    stdio_connection = next((conn for conn in server_details.connections if conn.type == "stdio"), None)
    if not stdio_connection:
        raise ValueError("No STDIO connection found")

    from ..utils.config import build_runtime_args_and_env
    # 运行阶段：根据元数据和 config 组装最终 args/env
    final_args, final_env = build_runtime_args_and_env(stdio_connection, config)
    command = stdio_connection.stdioFunction[0] if stdio_connection.stdioFunction else "python"
    env = get_runtime_environment(final_env)
    verbose(f"Using environment: {json.dumps({k: '***' if k.lower().endswith('key') else v for k, v in env.items()})}")
    verbose(f"Executing: {command} {' '.join(final_args)}")

    try:
        # 创建服务器进程
        server_params = StdioServerParameters(
            command=command,
            args=final_args,
            env=env,
            encoding="utf-8"
        )

        verbose(f"Setting up stdio proxy client for {server_details.qualifiedName}")
        async with stdio_client(server_params, errlog=sys.stderr) as (read_stream, write_stream):
            verbose("Stdio proxy client connection established")

            # 查找服务器进程ID - 优先使用父子进程关系
            from ..utils.process_utils import find_server_process_by_command, save_pid_to_file, find_server_process_from_current

            # 首先尝试通过父子进程关系查找
            is_found_from_current, current_pid, cmd_str = find_server_process_from_current()
            if is_found_from_current and current_pid:
                verbose(f"[Runner] 通过父子进程关系找到服务器进程，PID: {current_pid}")
                server_pid = current_pid
            else:
                # 如果父子进程关系查找失败，尝试通过命令行查找
                server_pid = find_server_process_by_command(command, final_args)

            if server_pid:
                verbose(f"[Runner] 已找到MCP服务器进程，PID: {server_pid}")

                # 将进程ID写入PID文件
                # 尝试获取第四级进程作为客户端标识
                client_name = "unknown_client"
                try:
                    import psutil
                    current_process = psutil.Process()

                    # 收集完整的进程链
                    process_chain = []
                    process = current_process
                    while process:
                        process_chain.append(process.name())
                        process = process.parent()

                    # 打印完整进程链用于调试
                    verbose(f"[Runner] 完整进程链: {' -> '.join(reversed(process_chain))}")

                    # 尝试获取第四级进程（如果存在）
                    if len(process_chain) >= 3:
                        client_name = process_chain[2]
                        verbose(f"[Runner] 使用第三级进程作为客户端: {client_name}")
                    else:
                        # 如果进程链不够长，使用最后一个进程（顶层进程）
                        client_name = process_chain[-1] if process_chain else "unknown_client"
                        verbose(f"[Runner] 进程链不够长，使用顶层进程作为客户端: {client_name}")
                except Exception as e:
                    verbose(f"[Runner] 获取客户端进程名称失败: {e}，使用默认值: {client_name}")

                verbose(f"[Runner] 使用 '{client_name}' 作为客户端标识")
                pid_file_path = save_pid_to_file(server_pid, server_details.qualifiedName,
                                                 client_name, command, final_args)
                if pid_file_path:
                    verbose(f"[Runner] 已将服务器PID信息安全地保存到: {pid_file_path}")
            else:
                verbose(f"[Runner] 未能找到MCP服务器进程，将无法获取其PID")

            # 创建 MCP 客户端会话
            from mcp import ClientSession
            session = await exit_stack.enter_async_context(ClientSession(read_stream, write_stream))

            # 注册消息处理回调
            def handle_server_message(msg):
                verbose(f"[magenta][server][/magenta] {json.dumps(msg, ensure_ascii=False)}")
            session.on_message = handle_server_message

            # 初始化 MCP 协议
            if not await initialize_session(session):
                return

            # 使用简单的同步阻塞循环处理输入和服务器消息
            verbose("[Runner] 开始处理循环，使用同步阻塞模式")

            # 打印启动消息
            verbose("[cyan]MCP client running. Press Ctrl+C to stop.[/cyan]")

            # 获取事件循环并在主任务中添加信号处理程序
            loop = asyncio.get_event_loop()
            loop.add_signal_handler(signal.SIGINT, lambda: asyncio.create_task(handle_sigint()))

            # 循环处理客户端请求，直到关闭
            while not is_shutting_down:
                try:
                    # 从标准输入读取一行 (同步阻塞)
                    line = sys.stdin.readline()
                    if not line:
                        verbose("[Runner] 标准输入关闭，结束处理")
                        break

                    # 处理客户端请求
                    message = json.loads(line)
                    verbose(f"[stdin] Received message: {line.strip()}")

                    method = message.get("method", "")
                    # 根据消息类型处理
                    if "id" in message:  # 这是请求，需要响应
                        response = await process_client_request(message, session)
                        sys.stdout.write(response + "\n")
                        sys.stdout.flush()
                        verbose(f"[stdin] Response sent for method: {method}")
                    else:  # 这是通知，不需要响应
                        # 创建通知对象并发送
                        # notification_obj = create_request_object(message, method)
                        # await session.send_notification(notification_obj)
                        await session.send_notification(
                            types.ClientNotification(
                                types.InitializedNotification(method=method)
                            )
                        )
                        verbose(f"[stdin] Notification sent for method: {method}")

                    verbose(f"[stdin] Processed: {line.strip()}")

                except json.JSONDecodeError as e:
                    error(f"[stdin] JSON decode error: {e}")
                except Exception as e:
                    error(f"[stdin] Error processing input: {e}")
                    # 如果是请求(有ID)，才需要发送错误响应
                    try:
                        if 'message' in locals() and isinstance(message, dict) and "id" in message:
                            error_resp = json.dumps({
                                "jsonrpc": "2.0",
                                "id": message.get("id"),
                                "error": {
                                    "code": -32700,
                                    "message": f"Parse error: {str(e)}"
                                }
                            })
                            sys.stdout.write(error_resp + "\n")
                            sys.stdout.flush()
                            verbose(f"[stdin] Sent error response for parse error")
                    except Exception as err:
                        error(f"[stdin] Failed to send error response: {err}")

                # 检查是否有服务器消息需要处理
                # 注意：这部分仍需异步，因为我们需要非阻塞地检查服务器消息
                try:
                    # 使用超时机制非阻塞地检查服务器消息
                    with anyio.fail_after(0.1):  # 设置很短的超时
                        message = await read_stream.receive()
                        await handle_single_server_message(message)
                except TimeoutError:
                    # 超时表示没有消息，继续处理客户端请求
                    pass
                except Exception as e:
                    error(f"[Runner] 处理服务器消息异常: {e}")

            verbose("[Runner] 处理循环结束")

    except Exception as e:
        verbose(f"[red]Error running stdio proxy: {e}[/red]")
        raise
    finally:
        await cleanup()



import logging
import os
import tempfile
from datetime import datetime
from pathlib import Path
from typing import Any, Optional
from ..config.app_config import APP_ENV

# Get log directory based on environment


def get_log_directory():
    """
    根据环境变量和操作系统确定日志文件的保存位置

    根据不同环境选择不同的日志目录:
    - dev: 使用项目目录下的logs/cli
    - test/prod: 
      - Windows: 使用%APPDATA%\mcpdock\logs
      - macOS/Linux: 使用~/.mcpdock/logs
    - 如果无法创建上述目录，则使用系统临时目录

    Returns:
        日志目录路径
    """
    # 检测操作系统
    import sys
    is_windows = sys.platform == "win32"

    # 1. 根据环境变量选择合适的目录
    if APP_ENV == 'dev':
        # 开发环境：尝试使用项目目录
        try:
            current_path = Path(__file__).resolve()
            for parent in current_path.parents:
                if parent.name == 'cli':
                    log_dir = os.path.join(parent, 'logs', 'cli')
                    os.makedirs(log_dir, exist_ok=True)
                    return log_dir
            # 如果找不到cli目录，回退到上级目录
            project_dir = current_path.parents[2]  # 回退到上级目录
            log_dir = os.path.join(project_dir, 'logs', 'cli')
            os.makedirs(log_dir, exist_ok=True)
            return log_dir
        except Exception as e:
            print(f"Warning: Could not create log directory in project folder: {e}")

    # 2. 生产/测试环境：根据操作系统选择合适的目录
    try:
        if is_windows:
            # Windows环境：使用 APPDATA 目录
            app_data = os.environ.get('APPDATA', '')
            if app_data:
                log_dir = os.path.join(app_data, 'mcpdock', 'logs', 'cli')
            else:
                # 如果APPDATA不可用，回退到用户主目录
                home_dir = os.path.expanduser("~")
                log_dir = os.path.join(home_dir, 'mcpdock', 'logs', 'cli')
        else:
            # macOS/Linux环境：使用用户主目录下的.mcpdock文件夹
            home_dir = os.path.expanduser("~")
            log_dir = os.path.join(home_dir, '.mcpdock', 'logs', 'cli')

        os.makedirs(log_dir, exist_ok=True)
        return log_dir
    except Exception as e:
        print(f"Warning: Could not create log directory in user folder: {e}")

    # 3. 最终回退：使用系统临时目录
    try:
        temp_dir = tempfile.gettempdir()
        log_dir = os.path.join(temp_dir, 'mcpdock-logs', 'cli')
        os.makedirs(log_dir, exist_ok=True)
        return log_dir
    except Exception as e:
        print(f"Critical: Could not create log directory in temp folder: {e}")
        # 最后的回退方案：返回当前目录
        return '.'


# Set date format
date = datetime.now()
date_str = f"{date.year}-{date.month}-{date.day}"


def create_dir_if_not_exist(dir_name: str):
    """Create directory if it doesn't exist"""
    dir_path = os.path.join(get_log_directory(), dir_name)
    if not os.path.exists(dir_path):
        os.makedirs(dir_path, exist_ok=True)


# Define log formatter
class CustomFormatter(logging.Formatter):
    def __init__(self, worker_id: str = 'main'):
        super().__init__(fmt='[%(levelname)s] - [%(asctime)s] [%(worker_id)s] %(message)s',
                         datefmt='%Y-%m-%d %H:%M:%S.%f')
        self.worker_id = worker_id

    def format(self, record):
        record.worker_id = self.worker_id
        return super().format(record)


# Create CLI Logger
cli_logger = logging.getLogger('cli_logger')
cli_logger.setLevel(logging.DEBUG)

# Log file location
log_directory = get_log_directory()
cli_log_file = os.path.join(log_directory, f'{date_str}.log')

# Prevent duplicate logs if logger is imported multiple times
if not cli_logger.handlers:
    # File handler
    fh_cli = logging.FileHandler(cli_log_file, encoding='utf-8')
    fh_cli.setLevel(logging.DEBUG)
    fh_cli.setFormatter(CustomFormatter())
    cli_logger.addHandler(fh_cli)

    # Console handler: 仅在开发环境或DEBUG_STDIO_RUNNER=1时输出到终端
    # if APP_ENV == "dev":
    # ch_cli = logging.StreamHandler()
    # ch_cli.setLevel(logging.DEBUG)
    # ch_cli.setFormatter(CustomFormatter())
    # cli_logger.addHandler(ch_cli)


def verbose(message: str, err: Optional[Any] = False) -> None:
    """Logs a verbose message to both file and console (if INFO level)"""
    if err:
        cli_logger.error(message)
    else:
        cli_logger.debug(message)

# def verbose(message: str) -> None:
#     """Logs a verbose message to both file and console (if INFO level)"""
#     cli_logger.debug(message)


# For compatibility with existing code
debug = cli_logger.debug
info = cli_logger.info
warning = cli_logger.warning
error = cli_logger.error
critical = cli_logger.critical



"""
MCP 协议请求/响应处理工具模块

该模块提供了处理 MCP 协议请求和响应的工具函数，包括：
- 创建请求对象
- 发送请求并处理超时
- 处理服务器消息
"""
import json
import anyio
import traceback
from typing import Dict, Any, Optional
from logging import error
import sys

from ..utils.logger import verbose
from mcp.types import ClientRequest, ServerRequest
from pydantic import BaseModel

# 定义一个通用的 Model，用于接收任何 JSON 响应


class DictModel(BaseModel):
    @classmethod
    def model_validate(cls, value):
        if isinstance(value, dict):
            return value
        return dict(value)


def create_request_object(message: Dict[str, Any], method: str):
    """
    根据方法类型创建适当的请求对象

    Args:
        message: JSON-RPC 消息
        method: 请求方法名

    Returns:
        MCP 请求对象
    """
    # 作为代理，我们不需要严格验证方法是否符合标准列表
    # 直接创建ClientRequest对象，透明转发所有请求
    msg = dict(message)
    msg.pop("jsonrpc", None)  # 移除 jsonrpc 字段，SDK会自动添加
    msg.pop("id", None)       # 移除 id 字段，我们会在响应中重新添加

    try:
        # 尝试创建 ClientRequest
        return ClientRequest(**msg)
    except Exception as e:
        verbose('---------------------------------------')
        verbose(msg)
        verbose('---------------------------------------')
        # 如果创建失败，记录详细错误信息并回退到 ServerRequest
        verbose(f"[Runner] 创建 ClientRequest 失败，错误信息: {str(e)}，回退到 ServerRequest")
        return ServerRequest(method=method, params=message.get("params", {}))


async def send_request_with_timeout(session, req_obj, original_id, timeout_seconds=60):
    """
    发送请求并处理超时和错误情况

    Args:
        session: MCP 客户端会话
        req_obj: 请求对象
        original_id: 原始请求ID
        timeout_seconds: 超时时间(秒)

    Returns:
        JSON-RPC 响应字符串
    """
    try:
        # 初始化 resp 为 None，防止超时时未定义
        resp = None
        # 使用超时机制
        with anyio.move_on_after(timeout_seconds):
            # 记录请求信息
            verbose(f"[Runner] 发送请求，原始ID={original_id}, 方法={req_obj.method if hasattr(req_obj, 'method') else '未知'}")

            # 发送请求并等待响应
            resp = await session.send_request(req_obj, DictModel)

        if resp:
            # 直接使用原始ID构造响应
            return json.dumps({
                "id": original_id,
                "jsonrpc": "2.0",
                "result": resp
            })
        else:
            return json.dumps({
                "id": original_id,
                "jsonrpc": "2.0",
                "error": {
                    "code": -32000,
                    "message": "Request timed out or empty response"
                }
            })

    except Exception as e:
        # 处理请求错误
        error_msg = str(e)
        error_code = -32603

        # 分类错误类型
        if "timed out" in error_msg.lower():
            error_code = -32001
        elif "connection" in error_msg.lower():
            error_code = -32002

        return json.dumps({
            "id": original_id,
            "jsonrpc": "2.0",
            "error": {
                "code": error_code,
                "message": f"Request failed: {error_msg}"
            }
        })


async def process_client_request(message, session):
    """
    处理从客户端接收的请求并转发到服务器

    Args:
        message: 客户端请求消息
        session: MCP 客户端会话

    Returns:
        响应字符串
    """
    original_id = message.get("id")
    method = message.get("method")
    params = message.get("params", {})

    # 添加特殊处理，记录初始化请求的详细信息
    if method == "initialize":
        verbose(f"[Runner] 收到初始化请求 ID: {original_id}, 详细内容: {json.dumps(message)}")

        # 对初始化请求使用SDK的initialize方法，而不是简单转发
        try:
            verbose("[Runner] 使用SDK的initialize方法处理初始化请求")
            # 先创建正确的请求对象
            req_obj = create_request_object(message, method)

            # 发送实际的初始化请求到下游服务器
            verbose("[Runner] 向下游服务器发送初始化请求")
            init_result = await session.send_request(req_obj, DictModel)
            verbose(
                f"[Runner] 收到下游服务器初始化响应: {json.dumps(init_result)[:200] if isinstance(init_result, dict) else str(init_result)[:200]}...")

            # 确保我们有一个有效的响应
            if init_result is None:
                verbose("[Runner] 收到空的初始化响应，创建默认响应")
                init_result = {
                    "protocolVersion": "2024-11-05",
                    "serverInfo": {
                        "name": "mcpy-proxy",
                        "version": "1.0.0"
                    }
                }

            # 构造完整的JSON-RPC响应
            response = json.dumps({
                "jsonrpc": "2.0",
                "id": original_id,
                "result": init_result
            })

            verbose(f"[Runner] 构造的初始化响应: {response[:200]}...")
            return response
        except Exception as e:
            error(f"[Runner] 初始化请求处理失败: {str(e)}")
            error(f"[Runner] 异常堆栈: {traceback.format_exc()}")
            # 如果处理失败，回退到常规请求处理
            verbose("[Runner] 回退到常规请求处理方式")

    verbose(f"[stdin] Processing request with id: {original_id}, method: {method}")

    # 常规请求处理：确定请求类型和构建请求对象
    req_obj = create_request_object(message, method)

    # 发送请求并处理响应
    try:
        verbose(f"[Runner] 向下游服务器发送请求，method: {method}, id: {original_id}")
        result = await send_request_with_timeout(session, req_obj, original_id)

        if method == "initialize":
            verbose(f"[Runner] 收到初始化响应: {result}")

        return result
    except Exception as e:
        error(f"[Runner] 请求处理异常 ({method}): {str(e)}")
        raise


async def handle_single_server_message(data):
    """
    处理单个从服务器接收的消息并输出到标准输出

    Args:
        data: 服务器消息数据
    """
    try:
        # 记录接收到的原始数据类型，帮助调试
        verbose(f"[server_raw] Received data type: {type(data)}")

        # 尝试获取原始数据的字符串表示用于调试
        raw_data_str = str(data)
        if len(raw_data_str) > 500:
            raw_data_str = raw_data_str[:500] + "..."
        verbose(f"[server_raw] 原始数据: {raw_data_str}")

        # 根据数据类型进行处理
        if hasattr(data, "model_dump"):
            content = data.model_dump()
            verbose(f"[server_raw] Processed pydantic v2 model: {type(data)}")
        elif hasattr(data, "dict"):
            content = data.dict()
            verbose(f"[server_raw] Processed pydantic v1 model: {type(data)}")
        elif isinstance(data, dict):
            content = data
            verbose(f"[server_raw] Processed dict with {len(data)} keys")
        else:
            # 尝试转换为字符串，然后解析为JSON
            try:
                content = json.loads(str(data))
                verbose(f"[server_raw] Converted to JSON: {type(data)}")
            except:
                content = {"data": str(data)}
                verbose(f"[server_raw] Used raw string for unknown type: {type(data)}")

        # 检查是否是初始化响应
        is_init_response = False
        if isinstance(content, dict):
            if "result" in content and isinstance(content["result"], dict):
                result = content["result"]
                if "protocolVersion" in result or "serverInfo" in result:
                    is_init_response = True
                    verbose(f"[server] 检测到初始化响应: {json.dumps(content)[:200]}...")

                # 特别检查是否包含tools字段，这对于VSCode非常重要
                if "tools" in result:
                    verbose(f"[server] 检测到tools字段，工具数量: {len(result['tools'])}")

        # 检查数据是否已经是标准的 JSON-RPC 消息
        if isinstance(content, dict):
            if "jsonrpc" in content and ("id" in content or "method" in content):
                # 已经是标准格式，直接输出
                output = json.dumps(content)
                verbose(f"[server] Standard JSON-RPC message detected, id: {content.get('id')}")

            elif "result" in content and not "jsonrpc" in content:
                # 是结果但缺少 jsonrpc 字段，构造标准响应
                output = json.dumps({
                    "jsonrpc": "2.0",
                    "id": 1,  # 默认ID，应该不会被用到
                    "result": content["result"] if "result" in content else content
                })
                verbose(f"[server] Fixed response format by adding jsonrpc")

            else:
                # 其他类型的消息，包装为通知
                output = json.dumps(content)
                verbose("[server] Passing through data as-is")
        else:
            # 非字典类型，直接序列化
            output = json.dumps(content)

        # 写入 stdout 并立即刷新，确保 VS Code 能收到
        sys.stdout.write(output + "\n")
        sys.stdout.flush()
        verbose(f"[server] Response sent to stdout: {output}")

    except Exception as e:
        error(f"[server] Error processing server message: {e}")
        error(f"[server] 异常堆栈: {traceback.format_exc()}")
        # 尝试发送错误响应
        try:
            error_resp = json.dumps({
                "jsonrpc": "2.0",
                "id": 1,  # 使用默认ID
                "error": {
                    "code": -32603,
                    "message": f"Internal error: {str(e)}"
                }
            })
            sys.stdout.write(error_resp + "\n")
            sys.stdout.flush()
            verbose(f"[server] Sent error response due to: {e}")
        except:
            error("[server] Failed to send error response")


async def initialize_session(session):
    """
    初始化 MCP 协议会话，转发上游客户端的初始化请求到下游服务器

    Args:
        session: MCP 客户端会话

    Returns:
        初始化是否成功
    """
    try:
        # 关键点: 作为代理，我们不应该主动调用 session.initialize()
        # 上游客户端会发送初始化请求，我们应该在 handle_stdin 函数中处理
        verbose("[Runner] MCP 代理准备就绪，等待上游客户端的初始化请求...")
        return True
    except Exception as init_error:
        error_msg = f"代理初始化失败: {str(init_error)}"
        error(f"[Runner] {error_msg}")
        return False



"""
Utility to query if a process with a specific package name is running by checking its PID.
"""
import os

from cli.utils.logger import verbose
from ..utils.process_utils import read_pid_file


def query_pid_by_packagename(package_names: list) -> dict:
    """
    检查指定包名的进程是否存在于PID文件中。

    参数：
        package_names (list): 要查询的包名列表。

    返回：
        dict: 包含每个包名的PID文件数据（有值为数据，无则为None）。
    """
    if isinstance(package_names, str):
        package_names = [package_names]

    results = {}
    pid_data = read_pid_file()

    if not pid_data:
        verbose(f"未找到任何 PID 文件记录")
        for package_name in package_names:
            results[package_name] = None
        return results

    for package_name in package_names:
        value = None
        # 直接查找包名
        if package_name in pid_data and pid_data[package_name]:
            verbose(f"包名 '{package_name}' 的进程存在于PID文件中")
            value = pid_data[package_name]
        else:
            # 尝试部分匹配
            for server_name, entries in pid_data.items():
                if package_name in server_name and entries:
                    verbose(f"包名 '{package_name}' 匹配服务器 '{server_name}' 存在于PID文件中")
                    value = entries
                    break
            if value is None:
                verbose(f"未找到包名 '{package_name}' 的进程")
        results[package_name] = value
    return results



import click
import json
import asyncio
from typing import Dict, Any, Optional

from rich import print as verbose
from rich.console import Console

from ..utils.logger import verbose
from ..registry import resolve_package
from ..utils.config import choose_connection, format_run_config_values
from ..utils.runtime import check_and_notify_remote_server
from ..types.registry import RegistryServer, ConnectionDetails
from ..runners import create_stdio_runner, create_ws_runner, run_package_with_command, create_stream_http_runner


# These runner functions have been moved to their respective modules in the runners package

async def pick_server_and_run(
    server_details: RegistryServer,
    config: Dict[str, Any],
    api_key: Optional[str] = None,
    analytics_enabled: bool = False
) -> None:
    """
    Selects the appropriate runner and starts the server.
    """
    connection = choose_connection(server_details)

    if connection.type == "ws":
        if not connection.deploymentUrl:
            raise ValueError("Missing deployment URL")
        await create_ws_runner(connection.deploymentUrl, config, api_key)
    elif connection.type == "stdio":
        await create_stdio_runner(server_details, config, api_key, analytics_enabled)
    elif connection.type == "stream-http":
        await create_stream_http_runner(server_details, config, api_key, analytics_enabled)
    else:
        raise ValueError(f"Unsupported connection type: {connection.type}")


async def run_server(
    qualified_name: str,
    config: Dict[str, Any],
    api_key: Optional[str] = None
) -> None:
    """
    Runs a server with the specified configuration.

    The qualified_name can be in these formats:
    - Standard MCP server name: "company/package"
    - uv command: "uv:package_name [args]" 
    - npx command: "npx:package_name [args]"
    """
    try:
        # # Check if this is a special command format (uv: or npx:)
        # if qualified_name.startswith("uv:") or qualified_name.startswith("npx:"):
        #     command_parts = qualified_name.split(":", 1)
        #     command_type = command_parts[0].lower()
        #     package_spec = command_parts[1]

        #     # Extract package name and args if provided
        #     package_parts = package_spec.split(" ", 1)
        #     package_name = package_parts[0]
        #     args = package_parts[1].split() if len(package_parts) > 1 else []

        #     verbose(f"Running {command_type} command for package: {package_name}")
        #     await run_package_with_command(
        #         command_type,
        #         package_name,
        #         args,
        #         config,
        #         api_key
        #     )
        #     return

        # Initialize settings for regular MCP server
        verbose("Initializing runtime environment settings")

        # Resolve server package
        verbose(f"Resolving server package: {qualified_name}")
        resolved_server = await resolve_package(qualified_name)

        if not resolved_server:
            raise ValueError(f"Could not resolve server: {qualified_name}")

        # Format the final configuration with validation
        connection = choose_connection(resolved_server)
        validated_env = format_run_config_values(connection, config)
        # 合并 env 校验结果和原 config，优先保留 env 校验后的值
        final_config = {**config, **validated_env}

        # Inform about remote server if applicable
        # check_and_notify_remote_server(resolved_server)

        verbose(f"[blue][Runner] Connecting to server:[/blue] {resolved_server.qualifiedName}")
        verbose(f"Connection types: {[c.type for c in resolved_server.connections]}")

        # Assume analytics is disabled for now
        analytics_enabled = False

        # Run the server with the appropriate runner
        await pick_server_and_run(
            resolved_server,
            final_config,
            api_key,
            analytics_enabled
        )
    except Exception as e:
        verbose(f"[red][Runner] Fatal error:[/red] {str(e)}")
        raise


@click.command("run")
@click.argument("mcp_server", type=click.STRING)
@click.option("--config", "config_json", type=click.STRING, help="Provide JSON format configuration")
@click.option("--api-key", type=click.STRING, help="API key for retrieving saved configuration")
def run(mcp_server: str, config_json: Optional[str], api_key: Optional[str]):
    """Run an AI MCP server."""
    verbose(f"Attempting to run {mcp_server}...")
    server_config = None

    # Parse command line provided configuration
    if config_json:
        try:
            verbose(config_json)
            server_config = json.loads(config_json)
            verbose(f"Using provided config: {server_config}")
        except json.JSONDecodeError as e:
            verbose(f"Error: Invalid JSON provided for --config: {e}")
            return 1
    # If no config provided, use empty dict
    if server_config is None:
        verbose("No config provided, running with empty config")
        server_config = {}

    if api_key:
        verbose(f"API key provided: {'*' * len(api_key)}")

    try:
        # with console.status(f"Starting {mcp_server}...", spinner="dots") as status:
        # Run the server asynchronously
        verbose('running server....')
        asyncio.run(run_server(mcp_server, server_config, api_key))
        # status.update(f"Successfully started {mcp_server}")
    except Exception as e:
        verbose(f"Run failed: {str(e)}")
        return 1



import click
import json
import asyncio
from typing import Dict, Any, Optional

from rich import print as rprint
from rich.console import Console
from rich.spinner import Spinner

from ..utils.logger import verbose, info, debug, error, warning
from ..registry import resolve_package
from ..utils.runtime import ensure_uv_installed, ensure_bun_installed, check_and_notify_remote_server
from ..utils.config import choose_connection, collect_config_values, get_server_name, format_server_config, format_config_values
from ..config.client_config import read_config, write_config
from ..types.registry import ConfiguredServer, ClientConfig


async def validate_config_values(
    connection: Any,
    existing_values: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """
    Validates config values without prompting for user input.
    Only checks if required values are present.

    Args:
        connection: Connection details containing schema
        existing_values: Existing config values to validate

    Returns:
        Dictionary with validated config values

    Raises:
        ValueError: If required configuration values are missing
    """
    schema = connection.configSchema or {}
    env = schema.env if schema else {}
    if not env:
        return {}

    # Use existing values or empty dict
    config_values = existing_values or {}

    # 校验 env 的 required 字段
    missing_required = []
    for key, value in env.items():
        if getattr(value, "required", False):
            if key not in config_values or config_values[key] is None:
                missing_required.append(key)

    # 校验 args 的 required 字段（对象类型）
    args_schema = schema.args or []
    missing_args_required = []
    for arg in args_schema:
        if isinstance(arg, dict):
            name = arg.get("name")
            required = arg.get("required", False)
            if required and (not config_values.get(name)):
                missing_args_required.append(name)
    if missing_required or missing_args_required:
        missing_fields = ", ".join(missing_required + missing_args_required)
        raise ValueError(f"Missing required configuration values: {missing_fields}")

    # Format and return the validated config
    return format_config_values(connection, config_values)


async def build_final_args(connection: Any, config: Dict[str, Any]) -> list:
    """
    根据 configSchema.args 和 config，生成最终命令行参数列表。
    字符串直接加入，对象从 config 取值并拼接 --name=value。
    """
    schema = connection.configSchema or {}
    args_schema = schema.args or []
    final_args = []
    missing_required = []
    for arg in args_schema:
        if isinstance(arg, str):
            final_args.append(arg)
        elif isinstance(arg, dict):
            name = arg.get("name")
            required = arg.get("required", False)
            if name:
                value = config.get(name)
                if value is not None:
                    final_args.append(f"--{name}={value}")
                elif required:
                    missing_required.append(name)
    if missing_required:
        raise ValueError(f"Missing required args: {', '.join(missing_required)}")
    return final_args


async def install_mcp_server(
    package_identifier: str,
    target_client: Optional[str] = None,
    initial_config: Optional[Dict[str, Any]] = None,
    api_key: Optional[str] = None
) -> None:
    """
    Installs and configures an AI server package for a specified client.

    Args:
        package_identifier: The fully qualified name of the AI server package
        target_client: The AI client to configure the server for
        initial_config: Optional pre-defined configuration values
        api_key: Optional API key for fetching saved configurations
    """
    target_client = target_client or "claude"  # Default to Claude if no client specified
    verbose(f"Initiating installation of {package_identifier} for {target_client}")

    # Create and start spinner for package resolution
    try:
        # Resolve the package (fetch server metadata)
        verbose(f"Resolving package: {package_identifier}")
        server_data = await resolve_package(package_identifier)
        verbose(f"Package resolved successfully: {server_data.qualifiedName}")

        # Choose the appropriate connection type
        verbose("Choosing connection type...")
        connection = choose_connection(server_data)
        verbose(f"Selected connection type: {connection.type}")

        # Check for required runtimes and install if needed
        # Commented out as these are specific to JS environment
        # await ensure_uv_installed(connection)
        # await ensure_bun_installed(connection)

        # Inform users of remote server installation if applicable
        is_remote = check_and_notify_remote_server(server_data)
        if is_remote:
            verbose("Remote server detected, notification displayed")

        # Get the validated config values, no prompting
        verbose(f"Validating configuration for {package_identifier}...")
        collected_config_values = await validate_config_values(connection, initial_config)

        # Determine if we need to pass config flag
        config_flag_needed = initial_config is not None

        # 新增：处理 args
        verbose("Building final args from configSchema.args and config...")
        # final_args = await build_final_args(connection, initial_config or {})
        # verbose(f"Final args: {final_args}")

        # Format the server configuration
        verbose("Formatting server configuration...")
        server_config = format_server_config(
            package_identifier,
            collected_config_values,
            api_key,
            config_flag_needed
        )
        verbose(f"Formatted server config: {json.dumps(server_config.__dict__, indent=2)}")

        # Read existing config from client
        verbose(f"Installing for {target_client}...")
        verbose(f"Reading configuration for client: {target_client}")
        client_config = read_config(target_client)

        # Get normalized server name to use as key
        server_name = get_server_name(package_identifier)
        verbose(f"Normalized server ID: {server_name}")

        # Update client configuration with new server
        verbose("Updating client configuration...")
        if not isinstance(client_config.mcpServers, dict):
            # Initialize if needed
            client_config.mcpServers = {}

        # Add the new server config
        client_config.mcpServers[server_name] = server_config

        # Write updated configuration
        verbose("Writing updated configuration...")
        write_config(client_config, target_client)
        verbose("Configuration successfully written")

        rprint(f"[green]{package_identifier} successfully installed for {target_client}[/green]")

        # No prompt for client restart
        verbose("Installation completed successfully")

    except Exception as e:
        verbose(f"Installation error: {str(e)}")
        verbose(f"[red]Failed to install {package_identifier}[/red]")
        verbose(f"[red]Error: {str(e)}[/red]")
        raise e


@click.command("install")
@click.argument("mcp_server", type=click.STRING)
@click.option("--client", type=click.STRING, help="Specify the target AI client")
@click.option("--env", "env_json", type=click.STRING, help="Provide configuration as JSON (bypasses interactive prompts)")
@click.option("--api-key", type=click.STRING, help="Provide an API key for fetching saved configurations")
def install(mcp_server: str, client: Optional[str], env_json: Optional[str], api_key: Optional[str]):
    """Install an AI mcp-server with optional configuration."""
    verbose(f"Attempting to install {mcp_server}...")
    user_config = None
    if env_json:
        try:
            user_config = json.loads(env_json)
            verbose(f"Using provided config: {user_config}")
        except json.JSONDecodeError as e:
            verbose(f"Error: Invalid JSON provided for --config: {e}", err=True)
            return

    try:
        # Run the async installation process
        asyncio.run(install_mcp_server(mcp_server, client, user_config, api_key))
    except Exception as e:
        # 通过stderr输出结构化JSON格式的错误信息，便于上游进程捕获
        import sys
        import traceback
        error_obj = {
            "error_type": type(e).__name__,
            "message": str(e),
            "traceback": traceback.format_exc()
        }
        print(json.dumps(error_obj, ensure_ascii=False), file=sys.stderr)
        # 不直接sys.exit(1)，让上游进程有机会处理输出
        return 1
        # verbose(f"Installation failed: {str(e)}", err=True)
        # return 1



"""
Command Runners implementation for running Python or Node.js packages
"""
import json
import asyncio
import signal
import sys
import os
from typing import Dict, Any, Optional, List

from rich import print as rprint

from ..utils.logger import verbose


async def run_command_process(cmd: List[str], env: Optional[Dict[str, str]] = None) -> None:
    """
    Run a command as a subprocess and handle its output asynchronously

    Args:
        cmd: Command and arguments as a list
        env: Optional environment variables
    """
    verbose(f"Running command: {' '.join(cmd)}")

    process = await asyncio.create_subprocess_exec(
        *cmd,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE,
        env=env
    )

    # Setup signal handling for graceful shutdown
    def handle_sigint(sig, frame):
        rprint("[yellow]Received stop signal, terminating command...[/yellow]")
        process.terminate()
        sys.exit(0)

    signal.signal(signal.SIGINT, handle_sigint)

    # Process output in real-time
    async def read_stream(stream, prefix):
        while True:
            line = await stream.readline()
            if not line:
                break
            try:
                decoded_line = line.decode('utf-8').rstrip()
                if prefix == "stdout":
                    rprint(f"[blue]{decoded_line}[/blue]")
                else:
                    rprint(f"[yellow]{decoded_line}[/yellow]")
            except UnicodeDecodeError:
                rprint(f"[red]Error decoding output from {prefix}[/red]")

    # Create tasks for stdout and stderr
    stdout_task = asyncio.create_task(read_stream(process.stdout, "stdout"))
    stderr_task = asyncio.create_task(read_stream(process.stderr, "stderr"))

    # Wait for process to complete
    exit_code = await process.wait()

    # Wait for output to be fully processed
    await stdout_task
    await stderr_task

    if exit_code != 0:
        rprint(f"[red]Command exited with code {exit_code}[/red]")
    else:
        rprint(f"[green]Command completed successfully[/green]")


async def create_uv_runner(
    package_name: str,
    args: List[str],
    config: Dict[str, Any],
    api_key: Optional[str] = None
) -> None:
    """
    Creates a runner for executing Python packages with uv package manager

    Args:
        package_name: Name of the Python package to run
        args: Additional arguments to pass to the package
        config: Configuration options
        api_key: Optional API key for authentication
    """
    verbose(f"Starting uv runner for package: {package_name}")
    rprint(f"[blue]Running Python package with uv: {package_name}[/blue]")

    # Prepare environment variables from config
    env = os.environ.copy()
    for key, value in config.items():
        if isinstance(value, str):
            env[f"MCP_{key.upper()}"] = value
        else:
            env[f"MCP_{key.upper()}"] = json.dumps(value)

    if api_key:
        env["MCP_API_KEY"] = api_key

    # Build the uv command
    cmd = ["uv", "run", "-m", package_name]
    if args:
        cmd.extend(args)

    await run_command_process(cmd, env)


async def create_npx_runner(
    package_name: str,
    args: List[str],
    config: Dict[str, Any],
    api_key: Optional[str] = None
) -> None:
    """
    Creates a runner for executing Node.js packages with npx

    Args:
        package_name: Name of the Node.js package to run
        args: Additional arguments to pass to the package
        config: Configuration options
        api_key: Optional API key for authentication
    """
    verbose(f"Starting npx runner for package: {package_name}")
    rprint(f"[blue]Running Node.js package with npx: {package_name}[/blue]")

    # Prepare environment variables from config
    env = os.environ.copy()
    for key, value in config.items():
        if isinstance(value, str):
            env[f"MCP_{key.upper()}"] = value
        else:
            env[f"MCP_{key.upper()}"] = json.dumps(value)

    if api_key:
        env["MCP_API_KEY"] = api_key

    # Build the npx command
    cmd = ["npx", package_name]
    if args:
        cmd.extend(args)

    await run_command_process(cmd, env)


async def run_package_with_command(
    command_type: str,
    package_name: str,
    args: List[str],
    config: Dict[str, Any],
    api_key: Optional[str] = None
) -> None:
    """
    Run a package with the specified command runner (uv or npx)

    Args:
        command_type: Type of command runner to use ("uv" or "npx")
        package_name: Name of the package to run
        args: Additional arguments to pass to the package
        config: Configuration options
        api_key: Optional API key for authentication

    Raises:
        ValueError: If an unsupported command type is provided
    """
    if command_type.lower() == "uv":
        await create_uv_runner(package_name, args, config, api_key)
    elif command_type.lower() == "npx":
        await create_npx_runner(package_name, args, config, api_key)
    else:
        raise ValueError(f"Unsupported command type: {command_type}")



# config/app_config.py
import os
from dotenv import load_dotenv

# Load environment variables from a .env file
load_dotenv()


APP_NAME = os.getenv("APP_NAME", "My Application")
APP_VERSION = os.getenv("APP_VERSION", "1.0.0")
APP_ENV = os.getenv('ENV')
API_BASE = os.getenv('API_BASE')



import os
import json
import sys
import subprocess
from pathlib import Path
from typing import Dict, Any, List, Optional, Union, Literal, TypedDict

# Import types from the registry module
from ..types.registry import ClientConfig, ConfiguredServer
# Import logger
from ..utils.logger import verbose

# Define types for configuration targets


class ClientFileTarget(TypedDict):
    type: Literal["file"]
    path: Path


class ClientCommandTarget(TypedDict):
    type: Literal["command"]
    command: str
    # Potentially add args template if needed later


ClientInstallTarget = Union[ClientFileTarget, ClientCommandTarget]

# Determine platform-specific paths using pathlib
home_dir = Path.home()

if sys.platform == "win32":
    base_dir = Path(os.environ.get("APPDATA", home_dir / "AppData" / "Roaming"))
    vscode_storage_path = Path("Code") / "User" / "globalStorage"
    code_command = "code.cmd"
    code_insiders_command = "code-insiders.cmd"
elif sys.platform == "darwin":
    base_dir = home_dir / "Library" / "Application Support"
    vscode_storage_path = Path("Code") / "User" / "globalStorage"
    code_command = "code"
    code_insiders_command = "code-insiders"
else:  # Assume Linux/other Unix-like
    base_dir = Path(os.environ.get("XDG_CONFIG_HOME", home_dir / ".config"))
    vscode_storage_path = Path("Code/User/globalStorage")  # Note: Path combines these
    code_command = "code"
    code_insiders_command = "code-insiders"

default_claude_path = base_dir / "Claude" / "claude_desktop_config.json"

# Define client paths using platform-specific base directories
# Using lowercase keys for normalization
CLIENT_PATHS: Dict[str, ClientInstallTarget] = {
    "claude": {"type": "file", "path": default_claude_path},
    "trae": {"type": "file", "path": base_dir / "Trae" / "User" / "mcp.json"},
    "cline": {
        "type": "file",
        "path": base_dir / vscode_storage_path / "saoudrizwan.claude-dev" / "settings" / "cline_mcp_settings.json",
    },
    "roo-cline": {
        "type": "file",
        "path": base_dir / vscode_storage_path / "rooveterinaryinc.roo-cline" / "settings" / "cline_mcp_settings.json",
    },
    "windsurf": {"type": "file", "path": home_dir / ".codeium" / "windsurf" / "mcp_config.json"},
    "witsy": {"type": "file", "path": base_dir / "Witsy" / "settings.json"},
    "enconvo": {"type": "file", "path": home_dir / ".config" / "enconvo" / "mcp_config.json"},
    "cursor": {"type": "file", "path": home_dir / ".cursor" / "mcp.json"},
    "vscode": {"type": "command", "command": code_command},
    "vscode-insiders": {"type": "command", "command": code_insiders_command},
}


def get_config_target(client: Optional[str] = None) -> ClientInstallTarget:
    """Gets the configuration target (file path or command) for a given client."""
    normalized_client = (client or "claude").lower()
    verbose(f"Getting config target for client: {normalized_client}")

    target = CLIENT_PATHS.get(normalized_client)

    if target:
        verbose(f"Config target resolved to: {target}")
        return target
    else:
        # Fallback for unknown clients (similar to JS version)
        fallback_path = default_claude_path.parent.parent / (client or "claude") / f"{normalized_client}_config.json"
        fallback_target: ClientFileTarget = {"type": "file", "path": fallback_path}
        verbose(f"Client '{normalized_client}' not predefined, using fallback path: {fallback_target}")
        return fallback_target


def read_config(client: Optional[str] = None) -> ClientConfig:
    """Reads the configuration for the specified client."""
    client_name = (client or "claude").lower()
    verbose(f"Reading config for client: {client_name}")
    target = get_config_target(client_name)

    if target["type"] == "command":
        verbose(f"Client '{client_name}' uses command-based config. Reading not supported.")
        # Command-based installers don't support reading config back easily
        return ClientConfig(mcpServers={})

    config_path = target["path"]
    verbose(f"Checking if config file exists at: {config_path}")
    if not config_path.exists():
        verbose("Config file not found, returning default empty config.")
        return ClientConfig(mcpServers={})

    try:
        verbose("Reading config file content.")
        raw_content = config_path.read_text(encoding="utf-8")
        raw_config = json.loads(raw_content)
        verbose(f"Config loaded successfully: {json.dumps(raw_config, indent=2)}")

        # Ensure mcpServers key exists, return as ClientConfig object
        mcp_servers_data = raw_config.get("mcpServers", {})
        # Basic validation/conversion if needed, assuming structure matches ConfiguredServer for now
        configured_servers = {
            name: ConfiguredServer(**server_data)
            for name, server_data in mcp_servers_data.items()
            if isinstance(server_data, dict) and "command" in server_data and "args" in server_data
        }

        # Include other top-level keys from the config file
        other_config = {k: v for k, v in raw_config.items() if k != "mcpServers"}

        return ClientConfig(mcpServers=configured_servers, **other_config)

    except json.JSONDecodeError as e:
        verbose(f"Error decoding JSON from {config_path}: {e}")
        return ClientConfig(mcpServers={})
    except Exception as e:
        verbose(f"Error reading config file {config_path}: {e}")
        return ClientConfig(mcpServers={})


def _write_config_command(config: ClientConfig, target: ClientCommandTarget) -> None:
    """Writes configuration using a command (e.g., for VS Code)."""
    command = target["command"]
    args: List[str] = []

    # Convert ConfiguredServer back to dict for JSON stringify
    for name, server in config.mcpServers.items():
        server_dict = {"command": server.command, "args": server.args, "name": name}
        args.extend(["--add-mcp", json.dumps(server_dict)])

    full_command = [command] + args
    verbose(f"Running command: {' '.join(full_command)}")

    try:
        # Use shell=True cautiously if command might not be directly executable
        # Or better, ensure the command is in PATH
        result = subprocess.run(full_command, capture_output=True, text=True, check=True, encoding='utf-8')
        verbose(f"Executed command successfully. Output:\n{result.stdout}")
        if result.stderr:
            verbose(f"Command stderr:\n{result.stderr}")
    except FileNotFoundError:
        verbose(f"Error: Command '{command}' not found. Make sure it is installed and in your PATH.")
        raise FileNotFoundError(f"Command '{command}' not found.")
    except subprocess.CalledProcessError as e:
        verbose(f"Error executing command. Return code: {e.returncode}")
        verbose(f"Stdout:\n{e.stdout}")
        verbose(f"Stderr:\n{e.stderr}")
        raise RuntimeError(f"Command '{command}' failed: {e.stderr or e.stdout}")
    except Exception as e:
        verbose(f"An unexpected error occurred while running the command: {e}")
        raise


def _write_config_file(config: ClientConfig, target: ClientFileTarget) -> None:
    """Writes configuration to a file, merging with existing content."""
    config_path = target["path"]
    config_dir = config_path.parent

    verbose(f"Ensuring config directory exists: {config_dir}")
    config_dir.mkdir(parents=True, exist_ok=True)

    existing_config_data: Dict[str, Any] = {}
    if config_path.exists():
        try:
            verbose("Reading existing config file for merging.")
            existing_config_data = json.loads(config_path.read_text(encoding="utf-8"))
            verbose(f"Existing config loaded: {json.dumps(existing_config_data, indent=2)}")
        except json.JSONDecodeError as e:
            verbose(f"Error reading existing config file {config_path} for merge: {e}. Will overwrite.")
        except Exception as e:
            verbose(f"Error reading existing config file {config_path}: {e}. Will overwrite.")

    # Prepare the config to be written (convert dataclasses back to dicts)
    config_to_write = {
        **config.__dict__,  # Include other top-level keys if ClientConfig has them
        "mcpServers": {
            name: {"command": server.command, "args": server.args, "env": server.env}
            for name, server in config.mcpServers.items()
        }
    }

    # Merge configurations
    verbose("Merging configurations.")

    # Determine if this is an uninstall operation by checking for keys in existing_config that are missing in config_to_write
    existing_servers = existing_config_data.get("mcpServers", {})
    new_servers = config_to_write.get("mcpServers", {})
    is_uninstall = False

    if existing_servers and new_servers:
        # Check if some server keys in existing config are missing in new config
        deleted_servers = set(existing_servers.keys()) - set(new_servers.keys())
        is_uninstall = len(deleted_servers) > 0
        if is_uninstall:
            verbose(f"Detected uninstall operation. Servers to remove: {deleted_servers}")

    # If this is an uninstall, we should not merge old server configs back in
    if is_uninstall:
        merged_config = {
            **existing_config_data,
            **config_to_write,
            # For uninstall: Don't merge old server configs, only use the new (updated) config
            "mcpServers": new_servers,
        }
    else:
        # For normal install: Merge configurations, new configs have precedence
        merged_config = {
            **existing_config_data,
            **config_to_write,
            # Ensure mcpServers are merged correctly
            "mcpServers": {
                **(existing_config_data.get("mcpServers") or {}),
                **(config_to_write.get("mcpServers") or {}),
            }
        }
    verbose(f"Merged config: {json.dumps(merged_config, indent=2)}")

    try:
        verbose(f"Writing config to file: {config_path}")
        config_path.write_text(json.dumps(merged_config, indent=2), encoding="utf-8")
        verbose(f"Configuration successfully written to {config_path}")
    except Exception as e:
        verbose(f"Error writing config file {config_path}: {e}")
        raise IOError(f"Failed to write configuration to {config_path}") from e


def write_config(config: ClientConfig, client: Optional[str] = None) -> None:
    """Writes the configuration for the specified client, either to a file or via command."""
    client_name = (client or "claude").lower()
    verbose(f"Writing config for client: {client_name}")
    verbose(f"Config data: {config}")  # Dataclass repr is usually good

    if not isinstance(config, ClientConfig) or not isinstance(config.mcpServers, dict):
        verbose("Invalid config object provided.")
        raise TypeError("Invalid ClientConfig object provided to write_config")

    target = get_config_target(client_name)

    if target["type"] == "command":
        _write_config_command(config, target)
    else:
        # Prepare the configuration to be written
        cleaned_config = {
            **config.__dict__,
            "mcpServers": {
                name: server.__dict__ for name, server in config.mcpServers.items()
            }
        }
        # Deserialize back into ConfiguredServer objects after writing
        deserialized_config = ClientConfig(
            **{
                **cleaned_config,
                "mcpServers": {
                    name: ConfiguredServer(**server) for name, server in cleaned_config["mcpServers"].items()
                }
            }
        )
        _write_config_file(deserialized_config, target)
